package tools.descartes.modelextraction.jboss.extraction.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.sasl.RealmCallback;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.jboss.as.controller.client.ModelControllerClient;
import org.jboss.dmr.ModelNode;

import tools.descartes.librede.Librede;
import tools.descartes.librede.configuration.ConfigurationFactory;
import tools.descartes.librede.configuration.DataSourceConfiguration;
import tools.descartes.librede.configuration.FileTraceConfiguration;
import tools.descartes.librede.configuration.LibredeConfiguration;
import tools.descartes.librede.configuration.Parameter;
import tools.descartes.librede.configuration.SchedulingStrategy;
import tools.descartes.librede.configuration.Service;
import tools.descartes.librede.configuration.TraceToEntityMapping;
import tools.descartes.librede.metrics.StandardMetrics;
import tools.descartes.librede.units.Ratio;
import tools.descartes.librede.units.RequestRate;
import tools.descartes.librede.units.Time;
import tools.descartes.librede.units.UnitsFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ProvidingDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemFactory;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentContext;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ActiveResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.NumberOfParallelProcessingUnits;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ProcessingResourceSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ResourceconfigurationFactory;
import edu.kit.ipd.descartes.mm.resourceconfiguration.SchedulingPolicy;
import edu.kit.ipd.descartes.mm.resourcelandscape.ComputingInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.HardwareInfrastructure;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapeFactory;
import edu.kit.ipd.descartes.mm.resourcetype.ProcessingResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceType;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypePackage;

/**
 * Hello world!
 *
 */
public class App {

	private static final String APP_NAME = "testsystem";
	private static final String BASE_PATH = "testmodel";

	public static void main(String[] args) throws IOException {
		new App().run();
	}

	private void run() throws IOException {
		File baseDir = new File("C:\\Users\\Simon\\Workspaces\\specjent-model\\specjent-model3");

		String[] servers = new String[] { "10.0.1.155", "10.0.1.154", "10.0.1.153", "10.0.1.150",
				"10.0.1.152", "10.0.1.151" };

		ResourceSet rset = createResourceSet();

		ResourceTypeRepository resTypes = loadResourceTypes(rset);

		DistributedDataCenter dc = createResourceLandscape(resTypes, servers);
		registerWithResourceSet(rset, "testsystem.resourcelandscape", dc);
		saveModel(new File(baseDir, "testsystem.resourcelandscape"), dc);

		String resTypesString = serialize(resTypes);
		try (ModelControllerClient client = createClient(InetAddress.getByName("10.0.1.148"), 9999, "admin", "descartes", "ManagementRealm")) {
			Repository repo = RepositoryFactory.eINSTANCE.createRepository();
			repo.setName(APP_NAME);
			
			Interface sql = RepositoryFactory.eINSTANCE.createInterface();
			sql.setName("java.sql.Connection");
			Signature query = RepositoryFactory.eINSTANCE.createSignature();
			query.setName("query");
			
			BasicComponent database = RepositoryFactory.eINSTANCE.createBasicComponent();
			database.setName("mysql");
			InterfaceProvidingRole sqlRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
			sqlRole.setName("mysql#Prov:java.sql.Connection");
			sqlRole.setInterface(sql);
			database.getInterfaceProvidingRoles().add(sqlRole);
			
			repo.getInterfaces().add(sql);			
			repo.getComponents().add(database);

			registerWithResourceSet(rset, repo.getName() + ".repository", repo);

			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "prisma");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(serialize(repo));
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);

				ModelNode ret = client.execute(op);

				repo = deserializeRepository(repo.eResource(), ret.get("result").get(0).asString());
			}

			String repositoryAsString = serialize(repo);
			edu.kit.ipd.descartes.mm.applicationlevel.system.System system = SystemFactory.eINSTANCE.createSystem();
			system.setName(repo.getName());
			
			AssemblyContext mysqlCtx = RepositoryFactory.eINSTANCE.createAssemblyContext();
			mysqlCtx.setName("mysql");
			mysqlCtx.setEncapsulatedComponent(database);
			system.getAssemblyContexts().add(mysqlCtx);
			
			registerWithResourceSet(rset, system.getName() + ".system", system);
			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "prisma");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(repositoryAsString);
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);
				ModelNode systemParam = op.get("system");
				systemParam.set(serialize(system));

				ModelNode ret = client.execute(op);

				system = deserializeSystem(system.eResource(), ret.get("result").get(0).asString());
			}

			Set<String> externalInterfaces = new HashSet<>(Arrays.asList("org.spec.jent.ejb.mfg.session.WorkOrderService", "http:/specj/app", "http:/supplier/BuyerService", "http:/specj-mfg/WorkOrderSessionService/WorkOrderSession"));	
			// Expose all provided interfaces
			List<AssemblyContext> newContexts = new LinkedList<>();
			for (AssemblyContext ctx : system.getAssemblyContexts()) {
				if (ctx.getEncapsulatedComponent() != null) {
					for (InterfaceProvidingRole componentRole : ctx.getEncapsulatedComponent().getInterfaceProvidingRoles()) {
						if (externalInterfaces.contains(componentRole.getInterface().getName())) {
							InterfaceProvidingRole role = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
							role.setInterface(componentRole.getInterface());
							role.setId(system.getName() + "_" + componentRole.getInterface().getName().replace(":", "_").replace("/", "_"));
							role.setName(system.getName() + "#Prov:" + componentRole.getInterface().getName());
							system.getInterfaceProvidingRoles().add(role);
							
							BasicComponent loadBalancer = createFrontEndLoadBalancer(ctx, role.getInterface());
							repo.getComponents().add(loadBalancer);
							AssemblyContext loadBalancerContext = RepositoryFactory.eINSTANCE.createAssemblyContext();
							loadBalancerContext.setName(loadBalancer.getName());
							loadBalancerContext.setEncapsulatedComponent(loadBalancer);
							newContexts.add(loadBalancerContext);
		
							ProvidingDelegationConnector connector = RepositoryFactory.eINSTANCE
									.createProvidingDelegationConnector();
							connector.setAssemblyContext(loadBalancerContext);
							connector.setInnerInterfaceProvidingRole(loadBalancer.getInterfaceProvidingRoles().get(0));
							connector.setOuterInterfaceProvidingRole(role);
							connector.setName(role.getName() + " -> " + loadBalancerContext.getName());
							system.getProvidingDelegationConnectors().add(connector);
							
							AssemblyConnector outgoingConnector = RepositoryFactory.eINSTANCE.createAssemblyConnector();
							outgoingConnector.setName(loadBalancerContext.getName() + " -> " + ctx.getName());
							outgoingConnector.setProvidingAssemblyContext(ctx);
							outgoingConnector.setRequiringAssemblyContext(loadBalancerContext);
							outgoingConnector.setInterfaceRequiringRole(loadBalancer.getInterfaceRequiringRoles().get(0));
							outgoingConnector.setInterfaceProvidingRole(componentRole);
							system.getAssemblyConnectors().add(outgoingConnector);
						}
					}
				}
			}
			system.getAssemblyContexts().addAll(newContexts);
			saveModel(new File(baseDir, repo.getName() + ".repository"), repo);
			saveModel(new File(baseDir, system.getName() + ".system"), system);

			String systemAsString = serialize(system);
			Deployment deploy = DeploymentFactory.eINSTANCE.createDeployment();
			deploy.setName(APP_NAME);
			deploy.setSystem(system);
			deploy.setTargetResourceLandscape(dc);
			registerWithResourceSet(rset, deploy.getName() + ".deployment", deploy);
			for (String server : servers) {
				ModelNode op = new ModelNode();
				op.get("operation").set("extract-performance-model");

				ModelNode address = op.get("address");
				address.add("host", "slave-" + server);
				address.add("server", server);
				address.add("subsystem", "prisma");

				op.get("application").set(BASE_PATH + "/" + APP_NAME);
				op.get("host").set(server);
				ModelNode repositoryParam = op.get("repository");
				repositoryParam.set(repositoryAsString);
				ModelNode resTypesParam = op.get("resourcetypes");
				resTypesParam.set(resTypesString);
				ModelNode systemParam = op.get("system");
				systemParam.set(systemAsString);
				ModelNode resourceLandscapeParam = op.get("resourcelandscape");
				resourceLandscapeParam.set(serialize(dc));
				ModelNode deploymentParam = op.get("deployment");
				deploymentParam.set(serialize(deploy));

				ModelNode ret = client.execute(op);

				deploy = deserializeDeployment(deploy.eResource(), ret.get("result").get(0).asString());
			}
			saveModel(new File(baseDir, deploy.getName() + ".deployment"), deploy);
		}
	}

	private BasicComponent createFrontEndLoadBalancer(AssemblyContext ctx, Interface interf) {
		String name = "Loadbalancer:" + ctx.getName();
		BasicComponent loadBalancer = RepositoryFactory.eINSTANCE.createBasicComponent();
		loadBalancer.setName(name);
		InterfaceProvidingRole provRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
		provRole.setInterface(interf);
		provRole.setName(name + "#Prov:" + interf.getName());
		loadBalancer.getInterfaceProvidingRoles().add(provRole);
		InterfaceRequiringRole reqRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
		reqRole.setInterface(interf);
		reqRole.setName(name + "#Req:" + interf.getName());
		loadBalancer.getInterfaceRequiringRoles().add(reqRole);
		
		for (Signature sig : interf.getSignatures()) {
			FineGrainedBehavior behavior = ServicebehaviorFactory.eINSTANCE.createFineGrainedBehavior();
			behavior.setInterfaceProvidingRole(provRole);
			behavior.setDescribedSignature(sig);
			loadBalancer.getFineGrainedBehavior().add(behavior);
			
			ComponentInternalBehavior sequence = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
			behavior.setBehavior(sequence);
			
			ExternalCall call = ServicebehaviorFactory.eINSTANCE.createExternalCall();
			call.setName(sig.getName());
			call.setInterfaceRequiringRole(reqRole);
			call.setSignature(sig);
			ExternalCallAction callAction = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
			callAction.setExternalCall(call);
			sequence.getActions().add(callAction);
			
		}
		return loadBalancer;
	}
	
	private void registerWithResourceSet(ResourceSet rset, String file, EObject content) {
		Resource res = rset.createResource(URI.createPlatformResourceURI(BASE_PATH + "/" + file, false));
		res.getContents().add(content);
	}

	private void saveModel(File file, EObject obj) throws IOException {
		obj.eResource().save(new FileOutputStream(file), Collections.EMPTY_MAP);
	}

	private String serialize(EObject obj) {
		try (ByteArrayOutputStream out = new ByteArrayOutputStream()) {
			obj.eResource().save(out, Collections.EMPTY_MAP);
			return out.toString();
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	private edu.kit.ipd.descartes.mm.applicationlevel.system.System deserializeSystem(Resource resource, String repo) {
		return (edu.kit.ipd.descartes.mm.applicationlevel.system.System) deserialize(resource, repo);
	}

	private Repository deserializeRepository(Resource resource, String repo) {
		return (Repository) deserialize(resource, repo);
	}

	private Deployment deserializeDeployment(Resource resource, String deploy) {
		return (Deployment) deserialize(resource, deploy);
	}

	private DistributedDataCenter createResourceLandscape(ResourceTypeRepository resTypes, String[] servers)
			throws IOException {
		DistributedDataCenter ddc = ResourcelandscapeFactory.eINSTANCE.createDistributedDataCenter();
		ddc.setName("My Company");
		DataCenter dc = ResourcelandscapeFactory.eINSTANCE.createDataCenter();
		dc.setName("My Datacenter");
		dc.setBelongsTo(ddc);

		for (String server : servers) {
			ComputingInfrastructure infra = ResourcelandscapeFactory.eINSTANCE.createComputingInfrastructure();
			infra.setName(server);
			infra.setPartOf(dc);

			NumberOfParallelProcessingUnits cores = ResourceconfigurationFactory.eINSTANCE
					.createNumberOfParallelProcessingUnits();
			cores.setNumber(1);
			cores.setName("Cores");
			ProcessingResourceSpecification cpuSpec = ResourceconfigurationFactory.eINSTANCE
					.createProcessingResourceSpecification();
			cpuSpec.setNrOfParProcUnits(cores);
			cpuSpec.setActiveResourceType(getCPU(resTypes));
			cpuSpec.setName("CPU");
			cpuSpec.setSchedulingPolicy(SchedulingPolicy.PROCESSOR_SHARING);
			
			NumberOfParallelProcessingUnits links = ResourceconfigurationFactory.eINSTANCE
					.createNumberOfParallelProcessingUnits();
			links.setNumber(1);
			links.setName("Links");
			ProcessingResourceSpecification netSpec = ResourceconfigurationFactory.eINSTANCE
					.createProcessingResourceSpecification();
			netSpec.setNrOfParProcUnits(links);
			netSpec.setActiveResourceType(getDelay(resTypes));
			netSpec.setName("Network");
			netSpec.setSchedulingPolicy(SchedulingPolicy.DELAY);			

			ActiveResourceSpecification res = ResourceconfigurationFactory.eINSTANCE
					.createActiveResourceSpecification();
			res.getProcessingResourceSpecifications().add(cpuSpec);
			res.getProcessingResourceSpecifications().add(netSpec);
			res.setName(server + "-ActiveResources");
			infra.getConfigSpec().add(res);			
			
		}

		return ddc;
	}

	private EObject deserialize(Resource resource, String content) {
		try (ByteArrayInputStream out = new ByteArrayInputStream(content.getBytes())) {
			resource.unload();
			resource.load(out, Collections.EMPTY_MAP);
			return resource.getContents().get(0);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	private ResourceSet createResourceSet() {
		ResourcetypePackage.eINSTANCE.eClass();

		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map<String, Object> m = reg.getExtensionToFactoryMap();
		m.put("repository", new XMIResourceFactoryImpl());
		m.put("system", new XMIResourceFactoryImpl());
		m.put("resourcelandscape", new XMIResourceFactoryImpl());
		m.put("resourcetype", new XMIResourceFactoryImpl());
		m.put("deployment", new XMIResourceFactoryImpl());
		m.put("librede", new XMIResourceFactoryImpl());

		ResourceSet resSet = new ResourceSetImpl();
		return resSet;
	}

	private ProcessingResourceType getCPU(ResourceTypeRepository types) {
		for (ResourceType t : types.getResourceTypes()) {
			if (t.getName().equals("CPU")) {
				return (ProcessingResourceType) t;
			}
		}
		return null;
	}
	
	private ProcessingResourceType getDelay(ResourceTypeRepository types) {
		for (ResourceType t : types.getResourceTypes()) {
			if (t.getName().equals("DELAY")) {
				return (ProcessingResourceType) t;
			}
		}
		return null;
	}

	private ResourceTypeRepository loadResourceTypes(ResourceSet rset) throws IOException {
		Resource resource = rset.createResource(URI
				.createURI("pathmap://DML_APPLICATIONLEVEL_MODELS/defaultresourcetypes.resourcetype"));
		resource.load(getClass().getResourceAsStream("/library/defaultresourcetypes.resourcetype"),
				Collections.EMPTY_MAP);
		return (ResourceTypeRepository) resource.getContents().get(0);
	}

	private static ModelControllerClient createClient(final InetAddress host, final int port, final String username,
			final String password, final String securityRealmName) {

		final CallbackHandler callbackHandler = new CallbackHandler() {

			public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
				for (Callback current : callbacks) {
					if (current instanceof NameCallback) {
						NameCallback ncb = (NameCallback) current;
						ncb.setName(username);
					} else if (current instanceof PasswordCallback) {
						PasswordCallback pcb = (PasswordCallback) current;
						pcb.setPassword(password.toCharArray());
					} else if (current instanceof RealmCallback) {
						RealmCallback rcb = (RealmCallback) current;
						rcb.setText(rcb.getDefaultText());
					} else {
						throw new UnsupportedCallbackException(current);
					}
				}
			}
		};

		return ModelControllerClient.Factory.create(host, port, callbackHandler);
	}
}
