package tools.descartes.prisma.agent.wildfly.deployment;

import org.jboss.as.ejb3.deployment.EjbDeploymentAttachmentKeys;
import org.jboss.as.server.AbstractDeploymentChainStep;
import org.jboss.as.server.deployment.DeploymentPhaseContext;
import org.jboss.as.server.deployment.DeploymentUnit;
import org.jboss.as.server.deployment.DeploymentUnitProcessingException;
import org.jboss.as.server.deployment.DeploymentUnitProcessor;
import org.jboss.as.server.deployment.Phase;
import org.jboss.ejb.client.EJBClientContext;
import org.jboss.logging.Logger;

import tools.descartes.prisma.agent.wildfly.interceptors.EJBOutgoingCallInterceptor;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;


/**
 * An example deployment unit processor that does nothing. To add more deployment
 * processors copy this class, and add to the {@link AbstractDeploymentChainStep}
 * {@link tools.descartes.prisma.agent.wildfly.extension.SubsystemAdd#performBoottime(org.jboss.as.controller.OperationContext, org.jboss.dmr.ModelNode, org.jboss.dmr.ModelNode, org.jboss.as.controller.ServiceVerificationHandler, java.util.List)}
 *
 * @author <a href="simon.spinner@uni-wuerzburg.de">Simon Spinner</a>
 */
public class ClientInstrumentationDeploymentProcessor implements DeploymentUnitProcessor {

    Logger log = Logger.getLogger(ClientInstrumentationDeploymentProcessor.class);

    /**
     * See {@link Phase} for a description of the different phases
     */
    public static final Phase PHASE = Phase.INSTALL;

    /**
     * The relative order of this processor within the {@link #PHASE}.
     * The current number is large enough for it to happen after all
     * the standard deployment unit processors that come with JBoss AS.
     */
    public static final int PRIORITY = 0x4000;

    @Override
    public void deploy(DeploymentPhaseContext phaseContext) throws DeploymentUnitProcessingException {
    	DeploymentUnit deploymentUnit = phaseContext.getDeploymentUnit();
        EJBClientContext ctx = deploymentUnit.getAttachment(EjbDeploymentAttachmentKeys.EJB_CLIENT_CONTEXT);
        if (ctx != null) {
        	log.info("Register client-side interceptor for deployment unit " + deploymentUnit.getName());
			ctx.registerInterceptor(Integer.MAX_VALUE, new EJBOutgoingCallInterceptor(MonitoringService.INSTANCE));
        }
    }

    @Override
    public void undeploy(DeploymentUnit context) {
    }

}
