package tools.descartes.prisma.agent.wildfly.filters;

import java.util.Arrays;
import java.util.List;

import org.jboss.as.controller.AbstractAddStepHandler;
import org.jboss.as.controller.AttributeDefinition;
import org.jboss.as.controller.OperationContext;
import org.jboss.as.controller.OperationFailedException;
import org.jboss.as.controller.PathAddress;
import org.jboss.as.controller.ServiceVerificationHandler;
import org.jboss.as.controller.descriptions.ModelDescriptionConstants;
import org.jboss.dmr.ModelNode;
import org.jboss.msc.service.ServiceController;
import org.jboss.msc.service.ServiceTarget;

import tools.descartes.prisma.agent.wildfly.service.FilterService;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

class FilterAdd extends AbstractAddStepHandler {
	
	public static final FilterAdd INSTANCE = new FilterAdd();
	
	private FilterAdd() {
		super(Arrays.asList(FilterDefinition.COMPONENT_PATTERN, FilterDefinition.INTERFACE_PATTERN, FilterDefinition.OPERATION_PATTERN));
	}
	
	@Override
	protected void performRuntime(OperationContext context, ModelNode operation, ModelNode model,
			ServiceVerificationHandler verificationHandler, List<ServiceController<?>> newControllers)
					throws OperationFailedException {
		final PathAddress address = PathAddress.pathAddress(operation.get(ModelDescriptionConstants.OP_ADDR));
		String name = address.getLastElement().getValue();
		
		
		final FilterService service = new FilterService(getResolvedModel(context, model));
		final ServiceTarget target = context.getServiceTarget();
		ServiceController<?> sc = target.addService(MonitoringService.FILTER_NAME.append(name), service)
				.setInitialMode(ServiceController.Mode.ON_DEMAND)
				.install();
		if (newControllers != null) {
			newControllers.add(sc);
		}
	}
	
	private ModelNode getResolvedModel(OperationContext context, ModelNode model) throws OperationFailedException {
        ModelNode resolved = new ModelNode();
        for (AttributeDefinition attribute : attributes) {
            resolved.get(attribute.getName()).set(attribute.resolveModelAttribute(context, model));
        }
        return resolved;
    }

}