package tools.descartes.prisma.agent.wildfly.interceptors;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;
import java.sql.SQLException;
import java.util.Deque;
import java.util.Properties;

import org.apache.commons.lang.mutable.MutableLong;
import org.jboss.logging.Logger;

import com.mysql.jdbc.Connection;
import com.mysql.jdbc.ResultSetInternalMethods;
import com.mysql.jdbc.Statement;
import com.mysql.jdbc.StatementInterceptorV2;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class JDBCStatementInterceptor implements StatementInterceptorV2 {
	
	private static final Logger log = Logger.getLogger(EJBOutgoingCallInterceptor.class);
	
	private final ThreadMXBean threadBean;
	
	private final MonitoringService monitoringService;

	private static final ThreadLocal<MutableLong> invocationTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};
	
	private static final ThreadLocal<MutableLong> startCpuTime = new ThreadLocal<MutableLong>() {
		protected MutableLong initialValue() {
			return new MutableLong();
		}
	};
	
	public JDBCStatementInterceptor() {
		this.monitoringService = MonitoringService.INSTANCE;
		threadBean = ManagementFactory.getThreadMXBean();
		if (threadBean.isCurrentThreadCpuTimeSupported()) {
			threadBean.setThreadCpuTimeEnabled(true);
		}
	}

	@Override
	public void destroy() {
		
	}

	@Override
	public boolean executeTopLevelOnly() {
		return true;
	}

	@Override
	public void init(Connection arg0, Properties arg1) throws SQLException {
	
	}

	@Override
	public ResultSetInternalMethods preProcess(String sql, Statement interceptedStatemenet, Connection con) throws SQLException {	
		Deque<InvocationRecord> threadCallStack = monitoringService.getCallStack();
		InvocationRecord currentInvocation = threadCallStack.peek();
		if (currentInvocation != null) {
			String interf = "javax.sql.DataSource";
			String operation = "query";
			
			ComponentRecord crec = MonitoringService.INSTANCE.requireComponent(con.getHost(),
					"mysql", "", "database");
			crec.setComponentClass("javax.sql.DataSource");
			OperationRecord orec = currentInvocation.requireEndpoint(crec, interf, "jdbc",
					operation);
			
			threadCallStack.push(orec.startInvocation());
			
//			startCpuTime.get().setValue(threadBean.getCurrentThreadCpuTime());
			invocationTime.get().setValue(System.nanoTime());
		}
		return null;
	}

	@Override
	public ResultSetInternalMethods postProcess(String sql, Statement interceptedStatement, ResultSetInternalMethods res,
		Connection con, int warnings, boolean noIndexUsed, boolean noGoodIndexUsed, SQLException ex) throws SQLException {
	
		long endTime = System.nanoTime();
//		long endCpuTime = threadBean.getCurrentThreadCpuTime();
		Deque<InvocationRecord> threadCallStack = monitoringService.getCallStack();
		if (!threadCallStack.isEmpty()) {
			long executionTime = endTime - invocationTime.get().longValue();
//			long cpuTime = endCpuTime - startCpuTime.get().longValue();
			InvocationRecord invocation = threadCallStack.pop();
			invocation.finishInvocation(executionTime, 0, 0);
				
			if (log.isTraceEnabled()) {
				InvocationRecord caller = threadCallStack.peek();
				log.trace("Outgoing invocation (" + caller.getOperation().getInterfaceName() + ", "
						+ caller.getOperation().getOperationName() + ") -> ("
						+ invocation.getOperation().getInterfaceName() + ", "
						+ invocation.getOperation().getOperationName() + ", " + executionTime + "ms)");
			}
		}
		return null;
	}

}
