package tools.descartes.prisma.agent.wildfly.interceptors;

import java.lang.reflect.Method;

import javax.servlet.ServletRequest;
import javax.servlet.ServletRequestWrapper;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.xml.ws.handler.MessageContext;

import org.jboss.as.ee.component.ComponentDescription;
import org.jboss.as.webservices.metadata.model.POJOEndpoint;
import org.jboss.invocation.InterceptorContext;

import io.undertow.server.HttpServerExchange;
import io.undertow.servlet.spec.HttpServletRequestImpl;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class WebServiceCallInterceptor extends AbstractInterceptor {

	public WebServiceCallInterceptor(MonitoringService monitoringService, ComponentDescription component,
			POJOEndpoint endpoint, Method method) {
		super(monitoringService, endpoint.getUrlPattern(), component, component.getComponentClassName(), "http",
				method.getName());
	}

	@Override
	public Object processInvocation(InterceptorContext ctx) throws Exception {
		// Wildfly puts the original HTTP request
		ServletRequest request = (HttpServletRequest) ctx.getContextData().get(MessageContext.SERVLET_REQUEST);
		if (request instanceof HttpServletRequestWrapper) {
			// Apache CXF may put a wrapper around the original request
			request = ((ServletRequestWrapper) request).getRequest();
		}

		// We need to access the internal request to get all information we
		// need.
		HttpServerExchange exchange = null;
		if (request instanceof HttpServletRequestImpl) {
			exchange = ((HttpServletRequestImpl) request).getExchange();

			startInvocation(ctx);
		} else {
			log.warn("Could not cast web service servlet request to undertow implementation.");
		}

		try {
			return ctx.proceed();
		} finally {
			if (exchange != null) {
				// In case of one-way messages, wildfly already sends a response
				// to the caller before the web service returns. In this case we
				// want to measure the execution time here. Otherwise, do the
				// measurements in the servlet filter as this returns later and
				// should therefore provide more accurate numbers.
				if (exchange.isResponseComplete()) {
					finishInvocation(exchange.getRequestStartTime());
				}
			}
		}
	}
}
