package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSContext;
import javax.jms.JMSException;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class ConnectionFactoryDelegate implements QueueConnectionFactory, TopicConnectionFactory {
	
	private final String jndiDelegate;
	private ConnectionFactory delegate = null;
	
	public ConnectionFactoryDelegate(String jndiDelegate) {
		this.jndiDelegate = jndiDelegate;
	}

	public Connection createConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	public Connection createConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}

	public JMSContext createContext() {
		return new JMSContextDelegate(getDelegate().createContext());
	}

	public JMSContext createContext(String userName, String password) {
		return new JMSContextDelegate(getDelegate().createContext(userName, password));
	}

	public JMSContext createContext(String userName, String password, int sessionMode) {
		return new JMSContextDelegate(getDelegate().createContext(userName, password, sessionMode));
	}

	public JMSContext createContext(int sessionMode) {
		return new JMSContextDelegate(getDelegate().createContext(sessionMode));
	}
	
	@Override
	public TopicConnection createTopicConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	@Override
	public TopicConnection createTopicConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}

	@Override
	public QueueConnection createQueueConnection() throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection());
	}

	@Override
	public QueueConnection createQueueConnection(String userName, String password) throws JMSException {
		return new ConnectionDelegate(getDelegate().createConnection(userName, password));
	}
	
	private ConnectionFactory getDelegate() {
		if (delegate == null) {
			Context ctx;
			try {
				ctx = new InitialContext();
				delegate = (ConnectionFactory) ctx.lookup(jndiDelegate);
			} catch (NamingException e) {
				throw new RuntimeException(e);
			}			
		}
		return delegate;
	}

}
