package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import javax.jms.CompletionListener;
import javax.jms.Destination;
import javax.jms.JMSProducer;
import javax.jms.Message;

public class JMSProducerDelegate implements JMSProducer {

	private final JMSProducer delegate;

	public JMSProducerDelegate(JMSProducer delegate) {
		this.delegate = delegate;
	}

	public JMSProducer send(Destination destination, Message message) {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, message);
		return this;
	}

	public JMSProducer send(Destination destination, String body) {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, body);
		return this;
	}

	public JMSProducer send(Destination destination, Map<String, Object> body) {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, body);
		return this;
	}

	public JMSProducer send(Destination destination, byte[] body) {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, body);
		return this;
	}

	public JMSProducer send(Destination destination, Serializable body) {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, body);
		return this;
	}

	public JMSProducer setDisableMessageID(boolean value) {
		delegate.setDisableMessageID(value);
		return this;
	}

	public boolean getDisableMessageID() {
		return delegate.getDisableMessageID();
	}

	public JMSProducer setDisableMessageTimestamp(boolean value) {
		delegate.setDisableMessageTimestamp(value);
		return this;
	}

	public boolean getDisableMessageTimestamp() {
		return delegate.getDisableMessageTimestamp();
	}

	public JMSProducer setDeliveryMode(int deliveryMode) {
		delegate.setDeliveryMode(deliveryMode);
		return this;
	}

	public int getDeliveryMode() {
		return delegate.getDeliveryMode();
	}

	public JMSProducer setPriority(int priority) {
		delegate.setPriority(priority);
		return this;
	}

	public int getPriority() {
		return delegate.getPriority();
	}

	public JMSProducer setTimeToLive(long timeToLive) {
		delegate.setTimeToLive(timeToLive);
		return this;
	}

	public long getTimeToLive() {
		return delegate.getTimeToLive();
	}

	public JMSProducer setDeliveryDelay(long deliveryDelay) {
		delegate.setDeliveryDelay(deliveryDelay);
		return this;
	}

	public long getDeliveryDelay() {
		return delegate.getDeliveryDelay();
	}

	public JMSProducer setAsync(CompletionListener completionListener) {
		delegate.setAsync(completionListener);
		return this;
	}

	public CompletionListener getAsync() {
		return delegate.getAsync();
	}

	public JMSProducer setProperty(String name, boolean value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, byte value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, short value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, int value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, long value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, float value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, double value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, String value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer setProperty(String name, Object value) {
		delegate.setProperty(name, value);
		return this;
	}

	public JMSProducer clearProperties() {
		delegate.clearProperties();
		return this;
	}

	public boolean propertyExists(String name) {
		return delegate.propertyExists(name);
	}

	public boolean getBooleanProperty(String name) {
		return delegate.getBooleanProperty(name);
	}

	public byte getByteProperty(String name) {
		return delegate.getByteProperty(name);
	}

	public short getShortProperty(String name) {
		return delegate.getShortProperty(name);
	}

	public int getIntProperty(String name) {
		return delegate.getIntProperty(name);
	}

	public long getLongProperty(String name) {
		return delegate.getLongProperty(name);
	}

	public float getFloatProperty(String name) {
		return delegate.getFloatProperty(name);
	}

	public double getDoubleProperty(String name) {
		return delegate.getDoubleProperty(name);
	}

	public String getStringProperty(String name) {
		return delegate.getStringProperty(name);
	}

	public Object getObjectProperty(String name) {
		return delegate.getObjectProperty(name);
	}

	public Set<String> getPropertyNames() {
		return delegate.getPropertyNames();
	}

	public JMSProducer setJMSCorrelationIDAsBytes(byte[] correlationID) {
		delegate.setJMSCorrelationIDAsBytes(correlationID);
		return this;
	}

	public byte[] getJMSCorrelationIDAsBytes() {
		return delegate.getJMSCorrelationIDAsBytes();
	}

	public JMSProducer setJMSCorrelationID(String correlationID) {
		delegate.setJMSCorrelationID(correlationID);
		return this;
	}

	public String getJMSCorrelationID() {
		return delegate.getJMSCorrelationID();
	}

	public JMSProducer setJMSType(String type) {
		delegate.setJMSType(type);
		return this;
	}

	public String getJMSType() {
		return delegate.getJMSType();
	}

	public JMSProducer setJMSReplyTo(Destination replyTo) {
		delegate.setJMSReplyTo(replyTo);
		return this;
	}

	public Destination getJMSReplyTo() {
		return delegate.getJMSReplyTo();
	}
	
	
}
