package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import java.util.Deque;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.Topic;

import org.jboss.logging.Logger;

import tools.descartes.prisma.agent.wildfly.callstack.ComponentRecord;
import tools.descartes.prisma.agent.wildfly.callstack.InvocationRecord;
import tools.descartes.prisma.agent.wildfly.callstack.OperationRecord;
import tools.descartes.prisma.agent.wildfly.service.MonitoringService;

public class MessageInterceptor {
	
	protected static final Logger log = Logger.getLogger(MessageInterceptor.class);

	public static void intercept(Destination destination) {
		MonitoringService monitoringService = MonitoringService.INSTANCE;
		Deque<InvocationRecord> threadCallStack = monitoringService.getCallStack();
		InvocationRecord currentInvocation = threadCallStack.peek();
		if (currentInvocation != null) {
			String destinationName = null;
			String componentClass = null;
			try {
				if (destination instanceof Queue) {
					destinationName = ((Queue)destination).getQueueName();
					componentClass = "javax.jms.Queue";
				} else if (destination instanceof Topic) {
					destinationName = ((Topic)destination).getTopicName();
					componentClass = "javax.jms.Topic";
				}
				if (destinationName != null) {
					ComponentRecord crec = monitoringService.requireComponent("", "", "",
							destinationName);
					if (crec.getComponentClass() == null) {
						crec.setComponentClass(componentClass);
					}
					OperationRecord orec = currentInvocation.requireEndpoint(crec,
							"javax.jms.MessageProducer", "jms", "send");
					InvocationRecord outgoingInvocation = orec.startInvocation();
					outgoingInvocation.finishInvocation(0, 0, 0);

					if (log.isTraceEnabled()) {
						log.trace("Outgoing invocation (" + orec.getInterfaceName() + ", " + orec.getOperationName()
								+ ") -> (" + currentInvocation.getOperation().getInterfaceName() + ", "
								+ currentInvocation.getOperation().getOperationName() + ")");
					}
				}
			} catch (JMSException e) {
				log.error("Error intercepting message.", e);
			}
		} else {
			log.warn("No invocation record found.");
		}
	}
}
