package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import javax.jms.CompletionListener;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueSender;
import javax.jms.Topic;
import javax.jms.TopicPublisher;

public class MessageProducerDelegate implements QueueSender, TopicPublisher {

	private final Destination destination;
	private final MessageProducer delegate;
	
	public MessageProducerDelegate(Destination destination, MessageProducer delegate) {
		this.destination = destination;
		this.delegate = delegate;
	}

	public void setDisableMessageID(boolean value) throws JMSException {
		delegate.setDisableMessageID(value);
	}

	public boolean getDisableMessageID() throws JMSException {
		return delegate.getDisableMessageID();
	}

	public void setDisableMessageTimestamp(boolean value) throws JMSException {
		delegate.setDisableMessageTimestamp(value);
	}

	public boolean getDisableMessageTimestamp() throws JMSException {
		return delegate.getDisableMessageTimestamp();
	}

	public void setDeliveryMode(int deliveryMode) throws JMSException {
		delegate.setDeliveryMode(deliveryMode);
	}

	public int getDeliveryMode() throws JMSException {
		return delegate.getDeliveryMode();
	}

	public void setPriority(int defaultPriority) throws JMSException {
		delegate.setPriority(defaultPriority);
	}

	public int getPriority() throws JMSException {
		return delegate.getPriority();
	}

	public void setTimeToLive(long timeToLive) throws JMSException {
		delegate.setTimeToLive(timeToLive);
	}

	public long getTimeToLive() throws JMSException {
		return delegate.getTimeToLive();
	}

	public void setDeliveryDelay(long deliveryDelay) throws JMSException {
		delegate.setDeliveryDelay(deliveryDelay);
	}

	public long getDeliveryDelay() throws JMSException {
		return delegate.getDeliveryDelay();
	}

	public Destination getDestination() throws JMSException {
		return delegate.getDestination();
	}

	public void close() throws JMSException {
		delegate.close();
	}

	public void send(Message message) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message);
	}

	public void send(Message message, int deliveryMode, int priority, long timeToLive) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message, deliveryMode, priority, timeToLive);
	}

	public void send(Destination destination, Message message) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, message);
	}

	public void send(Destination destination, Message message, int deliveryMode, int priority, long timeToLive)
			throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, message, deliveryMode, priority, timeToLive);
	}

	public void send(Message message, CompletionListener completionListener) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message, completionListener);
	}

	public void send(Message message, int deliveryMode, int priority, long timeToLive,
			CompletionListener completionListener) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message, deliveryMode, priority, timeToLive, completionListener);
	}

	public void send(Destination destination, Message message, CompletionListener completionListener)
			throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, message, completionListener);
	}

	public void send(Destination destination, Message message, int deliveryMode, int priority, long timeToLive,
			CompletionListener completionListener) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(destination, message, deliveryMode, priority, timeToLive, completionListener);
	}

	@Override
	public Queue getQueue() throws JMSException {
		return (Queue)destination;
	}

	@Override
	public void send(Queue queue, Message message) throws JMSException {
		MessageInterceptor.intercept(queue);
		delegate.send(queue, message);
	}

	@Override
	public void send(Queue queue, Message message, int deliveryMode, int priority, long timeToLive)
			throws JMSException {
		MessageInterceptor.intercept(queue);
		delegate.send(queue, message, deliveryMode, priority, timeToLive);		
	}

	@Override
	public Topic getTopic() throws JMSException {
		return (Topic)destination;
	}

	@Override
	public void publish(Message message) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message);		
	}

	@Override
	public void publish(Message message, int deliveryMode, int priority, long timeToLive) throws JMSException {
		MessageInterceptor.intercept(destination);
		delegate.send(message, deliveryMode, priority, timeToLive);		
	}

	@Override
	public void publish(Topic topic, Message message) throws JMSException {
		MessageInterceptor.intercept(topic);
		delegate.send(topic, message);	
	}

	@Override
	public void publish(Topic topic, Message message, int deliveryMode, int priority, long timeToLive)
			throws JMSException {
		MessageInterceptor.intercept(topic);
		delegate.send(topic, message, deliveryMode, priority, timeToLive);		
	}

}
