package tools.descartes.prisma.agent.wildfly.interceptors.jms;

import java.io.Serializable;

import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

public class SessionDelegate implements TopicSession, QueueSession {
	
	private final Session delegate;

	public SessionDelegate(Session delegate) {
		this.delegate = delegate;
	}

	public BytesMessage createBytesMessage() throws JMSException {
		return delegate.createBytesMessage();
	}

	public MapMessage createMapMessage() throws JMSException {
		return delegate.createMapMessage();
	}

	public Message createMessage() throws JMSException {
		return delegate.createMessage();
	}

	public ObjectMessage createObjectMessage() throws JMSException {
		return delegate.createObjectMessage();
	}

	public ObjectMessage createObjectMessage(Serializable object) throws JMSException {
		return delegate.createObjectMessage(object);
	}

	public StreamMessage createStreamMessage() throws JMSException {
		return delegate.createStreamMessage();
	}

	public TextMessage createTextMessage() throws JMSException {
		return delegate.createTextMessage();
	}

	public TextMessage createTextMessage(String text) throws JMSException {
		return delegate.createTextMessage(text);
	}

	public boolean getTransacted() throws JMSException {
		return delegate.getTransacted();
	}

	public int getAcknowledgeMode() throws JMSException {
		return delegate.getAcknowledgeMode();
	}

	public void commit() throws JMSException {
		delegate.commit();
	}

	public void rollback() throws JMSException {
		delegate.rollback();
	}

	public void close() throws JMSException {
		delegate.close();
	}

	public void recover() throws JMSException {
		delegate.recover();
	}

	public MessageListener getMessageListener() throws JMSException {
		return delegate.getMessageListener();
	}

	public void setMessageListener(MessageListener listener) throws JMSException {
		delegate.setMessageListener(listener);
	}

	public void run() {
		delegate.run();
	}

	public MessageProducer createProducer(Destination destination) throws JMSException {
		return new MessageProducerDelegate(destination, delegate.createProducer(destination));
	}

	public MessageConsumer createConsumer(Destination destination) throws JMSException {
		return delegate.createConsumer(destination);
	}

	public MessageConsumer createConsumer(Destination destination, String messageSelector) throws JMSException {
		return delegate.createConsumer(destination, messageSelector);
	}

	public MessageConsumer createConsumer(Destination destination, String messageSelector, boolean noLocal)
			throws JMSException {
		return delegate.createConsumer(destination, messageSelector, noLocal);
	}

	public MessageConsumer createSharedConsumer(Topic topic, String sharedSubscriptionName) throws JMSException {
		return delegate.createSharedConsumer(topic, sharedSubscriptionName);
	}

	public MessageConsumer createSharedConsumer(Topic topic, String sharedSubscriptionName, String messageSelector)
			throws JMSException {
		return delegate.createSharedConsumer(topic, sharedSubscriptionName, messageSelector);
	}

	public Queue createQueue(String queueName) throws JMSException {
		return delegate.createQueue(queueName);
	}

	public Topic createTopic(String topicName) throws JMSException {
		return delegate.createTopic(topicName);
	}

	public TopicSubscriber createDurableSubscriber(Topic topic, String name) throws JMSException {
		return delegate.createDurableSubscriber(topic, name);
	}

	public TopicSubscriber createDurableSubscriber(Topic topic, String name, String messageSelector, boolean noLocal)
			throws JMSException {
		return delegate.createDurableSubscriber(topic, name, messageSelector, noLocal);
	}

	public MessageConsumer createDurableConsumer(Topic topic, String name) throws JMSException {
		return delegate.createDurableConsumer(topic, name);
	}

	public MessageConsumer createDurableConsumer(Topic topic, String name, String messageSelector, boolean noLocal)
			throws JMSException {
		return delegate.createDurableConsumer(topic, name, messageSelector, noLocal);
	}

	public MessageConsumer createSharedDurableConsumer(Topic topic, String name) throws JMSException {
		return delegate.createSharedDurableConsumer(topic, name);
	}

	public MessageConsumer createSharedDurableConsumer(Topic topic, String name, String messageSelector)
			throws JMSException {
		return delegate.createSharedDurableConsumer(topic, name, messageSelector);
	}

	public QueueBrowser createBrowser(Queue queue) throws JMSException {
		return delegate.createBrowser(queue);
	}

	public QueueBrowser createBrowser(Queue queue, String messageSelector) throws JMSException {
		return delegate.createBrowser(queue, messageSelector);
	}

	public TemporaryQueue createTemporaryQueue() throws JMSException {
		return delegate.createTemporaryQueue();
	}

	public TemporaryTopic createTemporaryTopic() throws JMSException {
		return delegate.createTemporaryTopic();
	}

	public void unsubscribe(String name) throws JMSException {
		delegate.unsubscribe(name);
	}

	@Override
	public QueueReceiver createReceiver(Queue queue) throws JMSException {
		return (QueueReceiver)delegate.createConsumer(queue);
	}

	@Override
	public QueueReceiver createReceiver(Queue queue, String messageSelector) throws JMSException {
		return (QueueReceiver)delegate.createConsumer(queue, messageSelector);
	}

	@Override
	public QueueSender createSender(Queue queue) throws JMSException {
		return new MessageProducerDelegate(queue, delegate.createProducer(queue));
	}

	@Override
	public TopicSubscriber createSubscriber(Topic topic) throws JMSException {
		return (TopicSubscriber)delegate.createConsumer(topic);
	}

	@Override
	public TopicSubscriber createSubscriber(Topic topic, String messageSelector, boolean noLocal) throws JMSException {
		return (TopicSubscriber)delegate.createConsumer(topic, messageSelector, noLocal);
	}

	@Override
	public TopicPublisher createPublisher(Topic topic) throws JMSException {
		return new MessageProducerDelegate(topic, delegate.createProducer(topic));
	}
}
