package tools.descartes.prisma.agent.wildfly.service;

import org.jboss.as.naming.ImmediateManagedReferenceFactory;
import org.jboss.as.naming.ManagedReferenceFactory;
import org.jboss.as.naming.ServiceBasedNamingStore;
import org.jboss.as.naming.deployment.ContextNames;
import org.jboss.as.naming.service.BinderService;
import org.jboss.logging.Logger;
import org.jboss.msc.service.Service;
import org.jboss.msc.service.ServiceName;
import org.jboss.msc.service.StartContext;
import org.jboss.msc.service.StartException;
import org.jboss.msc.service.StopContext;

import tools.descartes.prisma.agent.wildfly.interceptors.jms.ConnectionFactoryDelegate;

public class InstrumentationService implements Service<InstrumentationService> {
	
	private static final Logger log = Logger.getLogger(InstrumentationService.class);
	
	public static final ServiceName SERVICE_NAME = ServiceName.JBOSS.append("modelextraction", "instrumentation");
	
	@Override
	public InstrumentationService getValue() throws IllegalStateException, IllegalArgumentException {
		return this;
	}

	@Override
	public void start(StartContext ctx) throws StartException {
		log.info("Instrument JMS connection factory: java:/JmsXA");
		
//		final ContextNames.BindInfo bindInfo = ContextNames.bindInfoFor("java:jboss/DefaultJMSConnectionFactory");
		final ContextNames.BindInfo bindInfo = ContextNames.bindInfoFor("java:jboss/InterceptedJMSConnectionFactory");
	    final BinderService binderService = new BinderService(bindInfo.getBindName());
	    
	    ConnectionFactoryDelegate delegate = new ConnectionFactoryDelegate("java:/JmsXA");
//	    ConnectionFactoryDelegate delegate = new ConnectionFactoryDelegate("java:jboss/DefaultJMSConnectionFactory");
	     
	    final ManagedReferenceFactory managedReferenceFactory = new ImmediateManagedReferenceFactory(delegate);
	    ctx.getChildTarget()
	             .addService(bindInfo.getBinderServiceName(), binderService)
	             .addInjection(binderService.getManagedObjectInjector(), managedReferenceFactory)
	             .addDependency(bindInfo.getParentContextServiceName(),
	                     ServiceBasedNamingStore.class,
	                     binderService.getNamingStoreInjector())
	             .install();
		
				
		ctx.complete();
	}

	@Override
	public void stop(StopContext ctx) {
		// TODO Auto-generated method stub
		
	}
}
