package tools.descartes.prisma.core.scopes;

import static org.fest.assertions.api.Assertions.assertThat;

import org.eclipse.emf.ecore.resource.Resource;
import org.junit.Test;

import edu.kit.ipd.descartes.mm.adaptationpoints.AdaptationPointDescriptions;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.model.scope.ScopeConfiguration;
import tools.descartes.prisma.model.sensor.SensorRepository;

public abstract class ContainerScopeTest<T extends AgentScope> {

	protected String name;
	protected T scope;
	protected ModelRepository repository;
	protected MessageBus messageBus;

	@Test
	public void testFolder() {
		assertThat(repository.exists(scope.getFolder()));
	}

	@Test
	public void testSensorRepository() {
		Resource sensorRes = repository.getResource(scope.getFolder() + "/" + name + ".sensors");
		assertThat(sensorRes).isNotNull();
		assertThat(sensorRes.getContents()).hasSize(1);
		SensorRepository sensor = (SensorRepository) sensorRes.getContents().get(0);
		assertThat(sensor).isNotNull();
		assertThat(sensor.getSensors()).isEmpty();
	}

	@Test
	public void testAgentRepository() {
		Resource scopeRes = repository.getResource(scope.getFolder() + "/" + name + ".scope");
		assertThat(scopeRes).isNotNull();
		assertThat(scopeRes.getContents()).hasSize(1);
		ScopeConfiguration conf = (ScopeConfiguration) scopeRes.getContents().get(0);
		assertThat(conf).isNotNull();
		assertThat(conf.getAgents()).isEmpty();
	}

	@Test
	public void testAdaptationPoints() {
		Resource adaptPointsRes = repository.getResource(scope.getFolder() + "/" + name + ".adaptationpoints");
		assertThat(adaptPointsRes).isNotNull();
		assertThat(adaptPointsRes.getContents()).hasSize(1);
		AdaptationPointDescriptions adaptPoints = (AdaptationPointDescriptions) adaptPointsRes.getContents().get(0);
		assertThat(adaptPoints).isNotNull();
		assertThat(adaptPoints.getName()).isEqualTo(name);
		assertThat(adaptPoints.getAdaptationPoints()).isEmpty();
	}
}
