package tools.descartes.prisma.core.scopes;

import static org.fest.assertions.api.Assertions.assertThat;

import org.eclipse.emf.ecore.resource.Resource;
import org.junit.Before;
import org.junit.Test;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.TestMessageBus;
import tools.descartes.prisma.core.TestModelRepository;

public class SystemGlobalScopeTest {

	private static final String NAME = "global";

	private SystemGlobalScope scope;
	private ModelRepository repository;
	private MessageBus messageBus;

	@Before
	public void setUp() {
		messageBus = new TestMessageBus();
		repository = new TestModelRepository(messageBus);
		Scope scope = repository.getSystemScope();
		assertThat(scope).isInstanceOf(SystemGlobalScope.class);
		scope.create();
		this.scope = (SystemGlobalScope) scope;
	}

	@Test
	public void testFolder() {
		assertThat(scope.getBasePath()).isEqualTo("");
		assertThat(scope.getFolder()).isEqualTo(scope.getName());
		assertThat(repository.exists(scope.getFolder()));
	}

	@Test
	public void testComponentRepository() {
		Resource repositoryRes = repository.getResource(scope.getFolder() + "/" + NAME + ".repository");
		assertThat(repositoryRes).isNotNull();
		assertThat(repositoryRes.getContents()).hasSize(1);
		Repository compRepository = (Repository) repositoryRes.getContents().get(0);
		assertThat(compRepository).isNotNull();
		assertThat(compRepository.getName()).isEqualTo(NAME);
		assertThat(compRepository.getComponents()).isEmpty();
		assertThat(compRepository.getDatatypes()).isEmpty();
		assertThat(compRepository.getInterfaces()).isEmpty();
	}

	@Test
	public void testResourceLandscape() {
		Resource landscapeRes = repository.getResource(scope.getFolder() + "/" + NAME + ".resourcelandscape");
		assertThat(landscapeRes).isNotNull();
		assertThat(landscapeRes.getContents()).hasSize(1);
		DistributedDataCenter landscape = (DistributedDataCenter) landscapeRes.getContents().get(0);
		assertThat(landscape).isNotNull();
		assertThat(landscape.getName()).isEqualTo(NAME);
		assertThat(landscape.getConsistsOf()).isEmpty();
	}

	@Test
	public void testGlobalResources() {
		Resource resourceTypeRes = repository.getResource(scope.getFolder() + "/" + NAME + ".resourcetypes");
		assertThat(resourceTypeRes).isNotNull();
		assertThat(resourceTypeRes.getContents()).hasSize(1);
		ResourceTypeRepository resourceTypes = (ResourceTypeRepository) resourceTypeRes.getContents().get(0);
		assertThat(resourceTypes).isNotNull();
		assertThat(resourceTypes.getResourceTypes()).isNotEmpty();
	}
}
