package tools.descartes.prisma.core.util;

import static org.fest.assertions.api.Assertions.assertThat;

import java.util.Arrays;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.AbstractAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BranchAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BranchProbabilities;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;

public class DMLHelper {

	public static ComponentInternalBehavior componentInternalBehavior(AbstractAction... actions) {
		ComponentInternalBehavior behavior = ServicebehaviorFactory.eINSTANCE.createComponentInternalBehavior();
		behavior.getActions().addAll(Arrays.asList(actions));
		return behavior;
	}

	public static InternalAction internalAction(ResourceDemand... demands) {
		InternalAction action = ServicebehaviorFactory.eINSTANCE.createInternalAction();
		action.getResourceDemand().addAll(Arrays.asList(demands));
		return action;
	}

	public static ExternalCallAction externalCallAction(ExternalCall call) {
		ExternalCallAction action = ServicebehaviorFactory.eINSTANCE.createExternalCallAction();
		action.setExternalCall(call);
		return action;
	}

	public static LoopAction loopAction(ComponentInternalBehavior loopBehavior) {
		LoopAction action = ServicebehaviorFactory.eINSTANCE.createLoopAction();
		LoopIterationCount count = ServicebehaviorFactory.eINSTANCE.createLoopIterationCount();
		count.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
		action.setLoopIterationCount(count);
		action.setLoopBodyBehavior(loopBehavior);
		return action;
	}

	public static BranchAction branchAction(ComponentInternalBehavior... branchBehaviors) {
		BranchAction action = ServicebehaviorFactory.eINSTANCE.createBranchAction();
		BranchProbabilities prob = ServicebehaviorFactory.eINSTANCE.createBranchProbabilities();
		prob.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
		action.setBranchProbabilities(prob);
		action.getBranches().addAll(Arrays.asList(branchBehaviors));
		return action;
	}

	public static ResourceDemand resourceDemand(String name) {
		ResourceDemand demand = ServicebehaviorFactory.eINSTANCE.createResourceDemand();
		demand.setName(name);
		demand.setCharacterization(ModelVariableCharacterizationType.EMPIRICAL);
		return demand;
	}

	public static void assertEMFModel(EObject o1, EObject o2) {
		assertThat(new EcoreUtil.EqualityHelper().equals(o1, o2)).isTrue();
	}

}
