package tools.descartes.prisma.core;

import java.io.Closeable;
import java.io.IOException;

import org.eclipse.emf.cdo.net4j.CDONet4jSessionConfiguration;
import org.eclipse.emf.cdo.net4j.CDONet4jUtil;
import org.eclipse.emf.cdo.session.CDOSession;
import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.cdo.view.CDOView;
import org.eclipse.net4j.Net4jUtil;
import org.eclipse.net4j.connector.IConnector;
import org.eclipse.net4j.tcp.TCPUtil;
import org.eclipse.net4j.util.container.ContainerUtil;
import org.eclipse.net4j.util.container.IManagedContainer;
import org.eclipse.net4j.util.om.OMPlatform;
import org.eclipse.net4j.util.om.log.PrintLogHandler;
import org.eclipse.net4j.util.om.trace.PrintTraceHandler;

import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemPackage;
import edu.kit.ipd.descartes.mm.deployment.DeploymentPackage;

public class RepositoryConnection implements Closeable {
	
	private static final int DEFAULT_PORT = 2036;
	
	private IManagedContainer container;
	private IConnector connector;
	private CDOSession session;
	
	private final String host;
	private final int port;
	
	public RepositoryConnection(String host) {
		this(host, DEFAULT_PORT);
	}
	
	public RepositoryConnection(String host, int port) {
		super();
		this.host = host;
		this.port = port;
	}
	
	public CDOTransaction startTransaction() {
		return session.openTransaction();
	}
	
	public CDOView startView() {
		return session.openView();
	}

	public void open() {
		// Enable logging and tracing
	    OMPlatform.INSTANCE.setDebugging(false);
	    OMPlatform.INSTANCE.addLogHandler(PrintLogHandler.CONSOLE);
	    OMPlatform.INSTANCE.addTraceHandler(PrintTraceHandler.CONSOLE);
	    
	    // Prepare container
	    container = ContainerUtil.createContainer();
	    Net4jUtil.prepareContainer(container); // Register Net4j factories
	    TCPUtil.prepareContainer(container); // Register TCP factories
	    CDONet4jUtil.prepareContainer(container); // Register CDO factories
	    container.activate();
	    
	    // Create connector
	    connector = TCPUtil.getConnector(container, host + ":" + port); //$NON-NLS-1$
	    
	    // Create configuration
	    CDONet4jSessionConfiguration configuration = CDONet4jUtil.createNet4jSessionConfiguration();
	    configuration.setConnector(connector);
	    configuration.setRepositoryName("prisma"); //$NON-NLS-1$
	    
	    // Open session
	    session = configuration.openNet4jSession();
	    session.getPackageRegistry().putEPackage(RepositoryPackage.eINSTANCE);
	    session.getPackageRegistry().putEPackage(ServicebehaviorPackage.eINSTANCE);
	    session.getPackageRegistry().putEPackage(FunctionsPackage.eINSTANCE);
	    session.getPackageRegistry().putEPackage(SystemPackage.eINSTANCE);
	    session.getPackageRegistry().putEPackage(DeploymentPackage.eINSTANCE);
	}

	@Override
	public void close() throws IOException {
		session.close();
		connector.close();
		container.deactivate();		
	}

}
