package tools.descartes.prisma.core.adapter;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import edu.kit.ipd.descartes.core.CorePackage;
import edu.kit.ipd.descartes.identifier.IdentifierPackage;
import edu.kit.ipd.descartes.mm.adaptationpoints.AdaptationpointsPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ParameterdependenciesPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorPackage;
import edu.kit.ipd.descartes.mm.applicationlevel.system.SystemPackage;
import edu.kit.ipd.descartes.mm.containerrepository.ContainerrepositoryPackage;
import edu.kit.ipd.descartes.mm.deployment.DeploymentPackage;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ResourceconfigurationPackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapePackage;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypePackage;
import edu.kit.ipd.descartes.mm.runtimeenvironmentclasses.RuntimeenvironmentclassesPackage;
import edu.kit.ipd.descartes.mm.usageprofile.UsageProfilePackage;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.adapter.ModelSkeletonAdapter.TransactionContext;
import tools.descartes.prisma.core.agent.AgentController;
import tools.descartes.prisma.model.sensor.SensorPackage;
import tools.descartes.prisma.model.skeleton.SkeletonPackage;

public class ModelSkeletonAdapterFactory extends AdapterFactoryImpl {
	
	private static final Set<EPackage> coveredPackages = new HashSet<>();
	
	static {
		// DML packages
		coveredPackages.add(IdentifierPackage.eINSTANCE);
		coveredPackages.add(CorePackage.eINSTANCE);
		coveredPackages.add(AdaptationpointsPackage.eINSTANCE);
		coveredPackages.add(ContainerrepositoryPackage.eINSTANCE);
		coveredPackages.add(DeploymentPackage.eINSTANCE);
		coveredPackages.add(FunctionsPackage.eINSTANCE);
		coveredPackages.add(ParameterdependenciesPackage.eINSTANCE);
		coveredPackages.add(RepositoryPackage.eINSTANCE);
		coveredPackages.add(ResourceconfigurationPackage.eINSTANCE);
		coveredPackages.add(ResourcelandscapePackage.eINSTANCE);
		coveredPackages.add(ResourcetypePackage.eINSTANCE);
		coveredPackages.add(RuntimeenvironmentclassesPackage.eINSTANCE);
		coveredPackages.add(ServicebehaviorPackage.eINSTANCE);
		coveredPackages.add(SystemPackage.eINSTANCE);
		coveredPackages.add(UsageProfilePackage.eINSTANCE);
		
		// Prisma packages
		coveredPackages.add(SkeletonPackage.eINSTANCE);
		coveredPackages.add(SensorPackage.eINSTANCE);
	}
	
	private final ModelRepository repository;
	private final AgentController agent;
	private final TransactionContext transactionContext = new TransactionContext();

	public ModelSkeletonAdapterFactory(ModelRepository repository, AgentController agent) {
		this.repository = repository;
		this.agent = agent;
	}

	@Override
	public boolean isFactoryForType(Object type) {
		if (coveredPackages.contains(type)) {
			return true;
		}
		if (type instanceof EObject) {
			return coveredPackages.contains(((EObject)type).eClass().getEPackage());
		}
		return false;
	}
	
	@Override
	protected Adapter createAdapter(Notifier target) {
		ModelSkeletonAdapter newAdapter = new ModelSkeletonAdapter(repository, agent, transactionContext,
				((EObject) target).eClass());
		newAdapter.setTarget(target);
		return newAdapter;
	}

}
