package tools.descartes.prisma.core.impl;

import java.util.concurrent.TimeoutException;

import org.eclipse.emf.cdo.CDOObject;
import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.cdo.util.CommitException;
import org.eclipse.emf.ecore.EObject;

import tools.descartes.prisma.core.Transaction;
import tools.descartes.prisma.core.exceptions.LockTimeoutException;

public class CDOTransactionWrapper implements Transaction {

	private final static long LOCK_TIMEOUT_IN_MILLIS = 60 * 1000;

	private final CDOModelRepository repository;
	private final CDOTransaction transaction;

	protected CDOTransactionWrapper(CDOModelRepository repository, CDOTransaction transaction) {
		this.repository = repository;
		this.transaction = transaction;
	}

	@Override
	public void close() {
		transaction.close();
	}

	@Override
	public void commit() {
		try {
			transaction.commit();
		} catch(CommitException ex) {
			throw new tools.descartes.prisma.core.exceptions.CommitException(ex);
		}
	}

	@Override
	public void rollback() {
		transaction.rollback();
	}

	@Override
	public <T extends EObject> T attach(T object) {
		return transaction.getObject(object);
	}

	@Override
	public <T extends EObject> T detach(T object) {
		return repository.getReadOnlyView().getObject(object);
	}

	@Override
	public void lock(EObject object, LockType lock) {
		try {
			if (lock == LockType.WRITE) {
				((CDOObject) object).cdoWriteLock().lock(LOCK_TIMEOUT_IN_MILLIS);
			} else if (lock == LockType.READ) {
				((CDOObject) object).cdoReadLock().lock(LOCK_TIMEOUT_IN_MILLIS);
			}
		} catch (TimeoutException e) {
			throw new LockTimeoutException(e);
		}
	}

	protected CDOTransaction getWrappedTransaction() {
		return transaction;
	}
}
