package tools.descartes.prisma.core.scopes;

import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ControlFlowVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.InfluencingParameter;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.Relationship;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.BasicComponent;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ComposedProvidingRequiringEntity;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Interface;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceProvidingRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.InterfaceRequiringRole;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.ProvidingDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Repository;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RepositoryFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.RequiringDelegationConnector;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.SubSystem;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResponseTime;
import edu.kit.ipd.descartes.mm.applicationlevel.system.System;
import edu.kit.ipd.descartes.mm.deployment.Deployment;
import edu.kit.ipd.descartes.mm.deployment.DeploymentFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;
import edu.kit.ipd.descartes.mm.resourcetype.ResourceTypeRepository;
import edu.kit.ipd.descartes.mm.resourcetype.ResourcetypeFactory;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.Transaction;
import tools.descartes.prisma.core.adapter.ModelSkeletonAdapter;
import tools.descartes.prisma.core.adapter.ObjectMatcher;
import tools.descartes.prisma.core.agent.AgentController;
import tools.descartes.prisma.model.sensor.Sensor;
import tools.descartes.prisma.model.skeleton.ModelSkeleton;

public class ApplicationScope extends AgentScope {

	public static final String APPLICATION_SCOPE_TYPE = "application";

	private static final String REPOSITORY_SUFFIX = ".repository";
	private static final String RESOURCE_TYPES_SUFFIX = ".resourcetypes";
	private static final String DEPLOYMENT_SUFFIX = ".deployment";
	private static final String APPLICATION_SUFFIX = ".application";

	private Repository componentRepository;
	private ResourceTypeRepository applicationResourceTypes;
	private Deployment deployment;
	private SubSystem application;
	private AssemblyContext applicationContext;

	public ApplicationScope(String name, SystemGlobalScope systemScope, ModelRepository repository,
			MessageBus messageBus) {
		super(name, APPLICATION_SCOPE_TYPE, systemScope, repository, messageBus);
	}

	public String getRuntimeEnvironmentExchange() {
		return getName() + ".RuntimeEnvironment";
	}

	public String getInterfaceExchange() {
		return getName() + ".Interface";
	}

	public String getBasicComponentExchange() {
		return getName() + ".BasicComponent";
	}

	public String getComposedProvidingRequiringEntityExchange() {
		return getName() + ".ComposedProvidingRequiringEntity";
	}

	public String getAssemblyContextExchange() {
		return getName() + ".AssemblyContext";
	}

	public String getResponseTimeExchange() {
		return getName() + ".ResponseTime";
	}

	public String getControlFlowVariableExchange() {
		return getName() + ".ControlFlowVariable";
	}

	public String getResourceDemandExchange() {
		return getName() + ".ResourceDemand";
	}

	public String getRelationshipExchange() {
		return getName() + ".Relationship";
	}

	public String getInfluencingParameterExchange() {
		return getName() + ".InfluencingParameter";
	}

	public String getApplicationSensorExchange() {
		return getName() + ".ApplicationSensor";
	}

	public String getExternalComposedProvidingRequiringEntityExchange() {
		return getName() + ".ExternalComposedProvidingRequiringEntity";
	}

	public Repository getComponentRepository() {
		if (componentRepository == null) {
			componentRepository = (Repository) getModelRepository().getResource(getComponentRepositoryPath()).getContents()
					.get(0);
		}
		return componentRepository;
	}

	private String getComponentRepositoryPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + REPOSITORY_SUFFIX;
	}

	public ResourceTypeRepository getApplicationResourceTypes() {
		if (applicationResourceTypes == null) {
			applicationResourceTypes = (ResourceTypeRepository) getModelRepository()
					.getResource(getApplicationResourceTypesPath()).getContents().get(0);
		}
		return applicationResourceTypes;
	}

	private String getApplicationResourceTypesPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + RESOURCE_TYPES_SUFFIX;
	}

	public Deployment getDeployment() {
		if (deployment == null) {
			deployment = (Deployment) getModelRepository().getResource(getDeploymentPath()).getContents().get(0);
		}
		return deployment;
	}

	private String getDeploymentPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + DEPLOYMENT_SUFFIX;
	}

	public SubSystem getApplication() {
		if (application == null) {
			application = (SubSystem) getModelRepository().getResource(getApplicationPath()).getContents().get(0);
		}
		return application;
	}

	private String getApplicationPath() {
		return APPLICATION_SCOPE_TYPE + "/" + getName() + "/" + getName() + APPLICATION_SUFFIX;
	}

	private AssemblyContext getApplicationContext() {
		SubSystem app = getApplication();
		if (applicationContext == null) {
			for (AssemblyContext curContext : getSystemScope().getSystem().getAssemblyContexts()) {
				if (curContext.getEncapsulatedComponent().equals(app)) {
					applicationContext = curContext;
					break;
				}
			}
		}
		return applicationContext;
	}

	@Override
	protected void createResources(Transaction transaction) {
		super.createResources(transaction);
		
		DistributedDataCenter ddc = getSystemScope().getResourceLandscape();
		System containingSystem = getSystemScope().getSystem();
		
		Resource resourceTypesResource = getModelRepository().getOrCreateResource(transaction,
				getApplicationResourceTypesPath());
		if (resourceTypesResource.getContents().isEmpty()) {
			ResourceTypeRepository resourceTypes = ResourcetypeFactory.eINSTANCE.createResourceTypeRepository();
			resourceTypesResource.getContents().add(resourceTypes);
		}
		
		Resource applicationResource = getModelRepository().getOrCreateResource(transaction, getApplicationPath());
		if (applicationResource.getContents().isEmpty()) {
			SubSystem application = RepositoryFactory.eINSTANCE.createSubSystem();
			application.setName(getName());
			applicationResource.getContents().add(application);
			registerInGlobalScope(transaction, application);
		}

		Resource repositoryResource = getModelRepository().getOrCreateResource(transaction,
				getComponentRepositoryPath());
		if (repositoryResource.getContents().isEmpty()) {
			Repository repository = RepositoryFactory.eINSTANCE.createRepository();
			repository.setName(getName());
			repositoryResource.getContents().add(repository);
		}

		Resource deploymentResource = getModelRepository().getOrCreateResource(transaction, getDeploymentPath());
		if (deploymentResource.getContents().isEmpty()) {
			Deployment deployment = DeploymentFactory.eINSTANCE.createDeployment();
			deployment.setName(getName());
			deployment.setSystem(containingSystem);
			deployment.setTargetResourceLandscape(ddc);
			deploymentResource.getContents().add(deployment);
		}
	}

	private void registerInGlobalScope(Transaction transaction, SubSystem application) {
		Repository globalRepository = transaction.attach(getSystemScope().getComponentRepository());
		globalRepository.getComponents().add(application);
		System globalSystem = transaction.attach(getSystemScope().getSystem());
		
		applicationContext = RepositoryFactory.eINSTANCE.createAssemblyContext();
		applicationContext.setName(getName());
		applicationContext.setEncapsulatedComponent(application);
		globalSystem.getAssemblyContexts().add(applicationContext);
	}

	private void exposePublicProvidedInterfaces(Transaction transaction) {
		// Explose all application level interface providing roles to the outside
		// system so that they can be referenced in usage profiles
		System outerSystem = transaction.attach(getSystemScope().getSystem());
		for (InterfaceProvidingRole curRole : application.getInterfaceProvidingRoles()) {
			boolean found = false;
			for (InterfaceProvidingRole outerRole : outerSystem.getInterfaceProvidingRoles()) {
				if (ObjectMatcher.matches(curRole, outerRole)) {
					found = true;
					break;
				}
			}
			if (!found) {
				// Create a new interface providing role and add a providing
				// delegation connector to the inner one
				InterfaceProvidingRole outerRole = RepositoryFactory.eINSTANCE.createInterfaceProvidingRole();
				outerRole.setName(curRole.getName());
				outerRole.setInterface(curRole.getInterface());
				outerSystem.getInterfaceProvidingRoles().add(outerRole);
				ProvidingDelegationConnector connector = RepositoryFactory.eINSTANCE
						.createProvidingDelegationConnector();
				connector.setName(curRole.getName());
				connector.setInnerInterfaceProvidingRole(curRole);
				connector.setOuterInterfaceProvidingRole(outerRole);
				connector.setAssemblyContext(getApplicationContext());
				outerSystem.getProvidingDelegationConnectors().add(connector);
			}
		}
	}

	private void exposePublicRequiredInterfaces(Transaction transaction) {
		// Explose all application level interface providing roles to the
		// outside system so that they can be referenced in usage profiles
		System outerSystem = transaction.attach(getSystemScope().getSystem());
		for (InterfaceRequiringRole curRole : application.getInterfaceRequiringRoles()) {
			boolean found = false;
			for (InterfaceRequiringRole outerRole : outerSystem.getInterfaceRequiringRoles()) {
				if (ObjectMatcher.matches(curRole, outerRole)) {
					found = true;
					break;
				}
			}
			if (!found) {
				// Create a new interface requiring role and add a requiring
				// delegation connector to the inner one
				InterfaceRequiringRole outerRole = RepositoryFactory.eINSTANCE.createInterfaceRequiringRole();
				outerRole.setName(curRole.getName());
				outerRole.setInterface(curRole.getInterface());
				outerSystem.getInterfaceRequiringRoles().add(outerRole);
				RequiringDelegationConnector connector = RepositoryFactory.eINSTANCE
						.createRequiringDelegationConnector();
				connector.setName(curRole.getName());
				connector.setInnerInterfaceRequiringRole(curRole);
				connector.setOuterInterfaceRequiringRole(outerRole);
				connector.setAssemblyContext(getApplicationContext());
				outerSystem.getRequiringDelegationConnectors().add(connector);
			}
		}
	}

	@Override
	public List<ModelSkeletonAdapter> synchronizeModelSkeletonTree(AgentController agent, Transaction transaction,
			ModelSkeleton skeleton) {
		List<ModelSkeletonAdapter> synchronizedAdapters = super.synchronizeModelSkeletonTree(agent, transaction,
				skeleton);
		if (skeleton.getRepository() != null) {
			ModelSkeletonAdapter adapter = agent.adapt(skeleton.getRepository(),
					ModelSkeletonAdapter.class);
			adapter.startTransaction(transaction);
			adapter.synchronizeRoot(getComponentRepository());
			synchronizedAdapters.add(adapter);
		}
		if (skeleton.getDeployment() != null) {
			ModelSkeletonAdapter adapter = agent.adapt(skeleton.getDeployment(),
					ModelSkeletonAdapter.class);
			adapter.startTransaction(transaction);
			adapter.synchronizeRoot(getDeployment());
			synchronizedAdapters.add(adapter);
		}
		if (skeleton.getSystem() != null) {
			ModelSkeletonAdapter adapter = agent.adapt(skeleton.getSystem(), ModelSkeletonAdapter.class);
			adapter.startTransaction(transaction);
			adapter.synchronizeRoot(getApplication());
			synchronizedAdapters.add(adapter);

			// IMPORTANT: wire the outer interface providing and requiring
			// roles up
			exposePublicProvidedInterfaces(transaction);
			exposePublicRequiredInterfaces(transaction);
		}
		return synchronizedAdapters;
	}

	@Override
	protected void createMessageExchanges() {
		super.createMessageExchanges();

		declareNotification(new NotificationDefinition(RuntimeEnvironment.class, getRuntimeEnvironmentExchange()));
		declareNotification(new NotificationDefinition(Interface.class, getInterfaceExchange()));
		declareNotification(
				new NotificationDefinition(BasicComponent.class, getBasicComponentExchange()));
		declareNotification(new NotificationDefinition(ComposedProvidingRequiringEntity.class,
				getComposedProvidingRequiringEntityExchange()));
		declareNotification(
				new NotificationDefinition(AssemblyContext.class, getAssemblyContextExchange()));
		declareNotification(
				new NotificationDefinition(ResponseTime.class, getResponseTimeExchange()));
		declareNotification(new NotificationDefinition(ControlFlowVariable.class, getControlFlowVariableExchange()));
		declareNotification(
				new NotificationDefinition(ResourceDemand.class, getResourceDemandExchange()));
		declareNotification(
				new NotificationDefinition(Relationship.class, getRelationshipExchange()));
		declareNotification(new NotificationDefinition(InfluencingParameter.class, getResourceDemandExchange()));
		declareNotification(
				new NotificationDefinition(Sensor.class, getApplicationSensorExchange()));
		declareNotification(new NotificationDefinition(ComposedProvidingRequiringEntity.class,
				getExternalComposedProvidingRequiringEntityExchange()));

		declareIncomingPort(getComposedProvidingRequiringEntityExchange());
		declareIncomingPort(getRuntimeEnvironmentExchange());
	}

}
