package tools.descartes.prisma.core.scopes;

import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.mm.resourcelandscape.ComputeNode;
import edu.kit.ipd.descartes.mm.resourcelandscape.DataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapeFactory;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;
import edu.kit.ipd.descartes.mm.resourcelandscape.StorageNode;
import edu.kit.ipd.descartes.mm.runtimeenvironmentclasses.RuntimeEnvironmentClasses;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.Transaction;
import tools.descartes.prisma.core.adapter.ModelSkeletonAdapter;
import tools.descartes.prisma.core.agent.AgentController;
import tools.descartes.prisma.model.skeleton.ModelSkeleton;

public class PhysicalInfrastructureScope extends InfrastructureScope {

	public static final String PHYSICAL_INFRASTRUCTURE_SCOPE_TYPE = "physical";

	private static final String DATACENTER_SUFFIX = ".datacenter";

	private DataCenter dataCenter;

	public PhysicalInfrastructureScope(String name, SystemGlobalScope systemScope, ModelRepository repository,
			MessageBus messageBus) {
		super(name, PHYSICAL_INFRASTRUCTURE_SCOPE_TYPE, systemScope, repository, messageBus);
	}

	public DataCenter getDataCenter() {
		if (dataCenter == null) {
			dataCenter = (DataCenter) getModelRepository().getResource(getDataCenterPath()).getContents().get(0);
		}
		return dataCenter;
	}

	public String getComputeNodeExchange() {
		return this.getName() + ".ComputeNode";
	}

	public String getStorageNodeExchange() {
		return this.getName() + ".StorageNode";
	}

	public String getHypervisorExchange() {
		return this.getName() + ".Hypervisor";
	}

	public String getDataCenterExchange() {
		return this.getName() + ".DataCenter";
	}

	protected String getDataCenterPath() {
		return PHYSICAL_INFRASTRUCTURE_SCOPE_TYPE + "/" + getName() + "/" + getName() + DATACENTER_SUFFIX;
	}

	@Override
	protected List<ModelSkeletonAdapter> synchronizeModelSkeletonTree(AgentController agent, Transaction transaction,
			ModelSkeleton skeleton) {
		List<ModelSkeletonAdapter> synchronizedAdapters = super.synchronizeModelSkeletonTree(agent, transaction,
				skeleton);

		if (skeleton.getDataCenter() != null) {
			ModelSkeletonAdapter adapter = agent.adapt(skeleton.getDataCenter(),
					ModelSkeletonAdapter.class);
			adapter.startTransaction(transaction);
			adapter.synchronizeRoot(getDataCenter());
			synchronizedAdapters.add(adapter);
		}
		return synchronizedAdapters;
	}

	@Override
	protected void createResources(Transaction transaction) {
		super.createResources(transaction);

		DistributedDataCenter ddc = getSystemScope().getResourceLandscape();

		Resource dataCenterResource = getModelRepository().getOrCreateResource(transaction, getDataCenterPath());
		if (dataCenterResource.getContents().isEmpty()) {
			DataCenter dataCenter = ResourcelandscapeFactory.eINSTANCE.createDataCenter();
			dataCenter.setName(getName());
			dataCenterResource.getContents().add(dataCenter);

			// Register this datacenter in the global system scope
			transaction.attach(ddc).getConsistsOf().add(dataCenter);
		}

	}

	@Override
	protected String getBasePath() {
		return PHYSICAL_INFRASTRUCTURE_SCOPE_TYPE;
	}

	@Override
	protected void createMessageExchanges() {
		super.createMessageExchanges();
		
		declareNotification(new NotificationDefinition(ComputeNode.class, getComputeNodeExchange()));
		declareNotification(new NotificationDefinition(StorageNode.class, getStorageNodeExchange()));
		declareNotification(new NotificationDefinition(DataCenter.class, getDataCenterExchange()));
		declareNotification(
				new NotificationDefinition(RuntimeEnvironment.class, getHypervisorExchange()) {
			@Override
			public boolean isActive(Object o) {
				if (super.isActive(o)) {
					return ((RuntimeEnvironment) o).getOfClass() == RuntimeEnvironmentClasses.HYPERVISOR;
				}
				return false;
			}
		});
	}

}
