package tools.descartes.prisma.core.scopes;

import edu.kit.ipd.descartes.mm.resourceconfiguration.ConfigurationSpecification;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;
import edu.kit.ipd.descartes.mm.runtimeenvironmentclasses.RuntimeEnvironmentClasses;
import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.model.sensor.Sensor;

public class PlatformScope extends ContainerScope {

	public static final String PLATFORM_SCOPE_TYPE = "platform";

	public PlatformScope(String name, SystemGlobalScope systemScope, ModelRepository repository,
			MessageBus messageBus) {
		super(name, PLATFORM_SCOPE_TYPE, systemScope, repository, messageBus);
	}

	public String getOperatingSystemExchange() {
		return getName() + ".OperatingSystem";
	}

	public String getRuntimeEnvironmentExchange() {
		return getName() + ".RuntimeEnvironment";
	}

	public String getConfigurationSpecificationExchange() {
		return getName() + ".ConfigurationSpecification";
	}

	public String getPlatformSensorExchange() {
		return getName() + ".PlatformSensor";
	}

	@Override
	protected void createMessageExchanges() {
		super.createMessageExchanges();

		declareNotification(
				new NotificationDefinition(RuntimeEnvironment.class, getOperatingSystemExchange()) {
					@Override
					public boolean isActive(Object o) {
						if (super.isActive(o)) {
							return ((RuntimeEnvironment) o).getOfClass() == RuntimeEnvironmentClasses.OPERATING_SYSTEM;
						}
						return false;
					}
				});
		declareNotification(new NotificationDefinition(RuntimeEnvironment.class, getRuntimeEnvironmentExchange()));
		declareNotification(new NotificationDefinition(ConfigurationSpecification.class,
				getConfigurationSpecificationExchange()));
		declareNotification(new NotificationDefinition(Sensor.class, getPlatformSensorExchange()));

		declareIncomingPort(getOperatingSystemExchange());
	}

}
