package tools.descartes.prisma.core.scopes;

import tools.descartes.prisma.core.MessageBus;
import tools.descartes.prisma.core.ModelRepository;
import tools.descartes.prisma.core.Transaction;
import tools.descartes.prisma.core.agent.AgentController;
import tools.descartes.prisma.core.exceptions.CommitException;
import tools.descartes.prisma.model.skeleton.ModelSkeleton;

public abstract class Scope {
	
	private final String name;
	private final ModelRepository repository;
	private final MessageBus messageBus;
	
	public Scope(String name, ModelRepository repository, MessageBus messageBus) {
		this.name = name;
		this.repository = repository;
		this.messageBus = messageBus;
	}
	
	public String getName() {
		return name;
	}
	
	public ModelRepository getModelRepository() {
		return repository;
	}
	
	public MessageBus getMessageBus() {
		return messageBus;
	}

	public String getFolder() {
		if (getBasePath().isEmpty()) {
			return getName();
		} else {
			return getBasePath() + "/" + getName();
		}
	}

	public boolean create() {
		try (Transaction transaction = getModelRepository().createTransaction()) {
			/*
			 * IMPORTANT: Lock the root object to avoid conflicts by agents who
			 * concurrently create scopes
			 */
			if (!getModelRepository().exists(getFolder())) {
				getModelRepository().createFolders(transaction, getFolder());
			}

			createResources(transaction);
			
			try {
				transaction.commit();
				// No initialize the message exchanges in the messaging
				// subsystem through which the coordination message are
				// transferred.
				createMessageExchanges();
				return true;
			} catch (CommitException e) {
				e.printStackTrace();
			}
			return false;
		}
	}

	public void waitForCreation() {
		ModelRepository repository = getModelRepository();
		while (!repository.exists(getFolder())) {
			repository.waitForUpdate();
		}
	}

	public abstract void synchronizeModelSkeleton(AgentController agent, Transaction transaction,
			ModelSkeleton skeleton);

	protected abstract void createResources(Transaction transaction);

	protected abstract String getBasePath();

	protected void createMessageExchanges() {
		// Do nothing;
	}
}
