package tools.descartes.prisma.core.util;

import org.eclipse.emf.cdo.CDOObject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import tools.descartes.prisma.core.adapter.ModelSkeletonAdapter;
import tools.descartes.prisma.core.adapter.ObjectMatcher;
import tools.descartes.prisma.core.agent.AgentController;

public class ModelSkeletonUtil {

	@SuppressWarnings("unchecked")
	public static <S extends T, T extends EObject> S find(EList<T> list, S prototype) {
		for (T existingObject : list) {
			if (ObjectMatcher.matches(existingObject, prototype)) {
				return (S) existingObject;
			}
		}
		return null;
	}

	public static <S extends T, T extends EObject> S insert(EList<T> list, S prototype) {
		S res = find(list, prototype);
		if (res == null) {
			list.add(prototype);
			return prototype;
		}
		return res;
	}

	public static <S extends T, T extends EObject> S adopt(AgentController agent,
			S repositoryObject) {
		EcoreUtil.Copier copier = new EcoreUtil.Copier();
		@SuppressWarnings("unchecked")
		S copyOfObject = (S) copier.copy(repositoryObject);
		copier.copyReferences();
		for (EObject curOriginal : copier.keySet()) {
			EObject curCopy = copier.get(curOriginal);
			ModelSkeletonAdapter skeletonAdapter = agent.adapt(curCopy, ModelSkeletonAdapter.class);
			skeletonAdapter.setRepositoryObject((CDOObject) curOriginal);
		}
		return copyOfObject;
	}

}
