package tools.descartes.prisma.parameterization;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;

import org.apache.log4j.BasicConfigurator;
import org.eclipse.emf.cdo.eresource.CDOResource;
import org.eclipse.emf.cdo.transaction.CDOTransaction;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

import edu.kit.ipd.descartes.mm.deployment.Deployment;
import tools.descartes.librede.Librede;
import tools.descartes.librede.configuration.ConfigurationFactory;
import tools.descartes.librede.configuration.EstimationAlgorithmConfiguration;
import tools.descartes.librede.configuration.EstimationApproachConfiguration;
import tools.descartes.librede.configuration.EstimationSpecification;
import tools.descartes.librede.configuration.LibredeConfiguration;
import tools.descartes.librede.configuration.ValidationSpecification;
import tools.descartes.librede.configuration.ValidatorConfiguration;
import tools.descartes.librede.registry.Registry;
import tools.descartes.librede.units.Time;
import tools.descartes.librede.units.UnitsFactory;
import tools.descartes.prisma.core.ApplicationModel;
import tools.descartes.prisma.core.RepositoryConnection;

public class Application implements IApplication {

	@Override
	public Object start(IApplicationContext context) throws Exception {
		BasicConfigurator.configure();
		
		Librede.init();
		
		RepositoryConnection repo = new RepositoryConnection("10.0.1.148");
		ApplicationModel model = new ApplicationModel(repo, "specjent2010");
		
		Path dataPath = Paths.get("D:/Dateien/VMware/SPECjExperiments/Run_2015-09-25_11-36-26");
		
		try {
			repo.open();
			
			CDOTransaction transaction = repo.startTransaction();
			
//			exportModel(transaction, model);
			
//			CDOResource res = transaction.getOrCreateResource(model.getDeploymentPath());
//			if (!res.getContents().isEmpty()) {
//				Path outputPath = Paths.get("C:/Users/Simon/Workspaces/runtime-Librede/prisma-output");		
//				Deployment deployment = (Deployment)res.getContents().get(0);
//				LibredeConfiguration conf = createLibredeConfiguration(deployment, dataPath);
//				
//				saveLibredeConfiguration(outputPath, conf);
//			}
			
			transaction.close();
		} finally {
			repo.close();
		}
		
		return null;
	}

	@Override
	public void stop() {
		// TODO Auto-generated method stub
	}
	
	private void exportModel(CDOTransaction transaction, ApplicationModel model) {
		String outputPath = "D:\\Dateien\\VMware\\SPECjExperiments\\PER-Experiments\\Model\\specj\\";
		
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
	    Map<String, Object> m = reg.getExtensionToFactoryMap();
	    m.put("resourcetype", new XMIResourceFactoryImpl());
	    m.put("resourcelandscape", new XMIResourceFactoryImpl());
	    m.put("repository", new XMIResourceFactoryImpl());
	    m.put("deployment", new XMIResourceFactoryImpl());
	    m.put("system", new XMIResourceFactoryImpl());
	    m.put("usageprofile", new XMIResourceFactoryImpl());
		
		Copier copier = new Copier();
		CDOResource res = transaction.getOrCreateResource(model.getResourceTypesPath());
		EObject restypes = copier.copy(res.getContents().get(0));
		res = transaction.getOrCreateResource(model.getResourceLandscapePath());
		EObject landscape = copier.copy(res.getContents().get(0));
		res = transaction.getOrCreateResource(model.getRepositoryPath());
		EObject repository = copier.copy(res.getContents().get(0));
		res = transaction.getOrCreateResource(model.getSystemPath());
		EObject system = copier.copy(res.getContents().get(0));
		res = transaction.getOrCreateResource(model.getDeploymentPath());
		EObject deployment = copier.copy(res.getContents().get(0));
		res = transaction.getOrCreateResource(model.getUsageProfilePath());
		EObject profile = copier.copy(res.getContents().get(0));
		copier.copyReferences();
		
		try {
			ResourceSet resSet = new ResourceSetImpl();
			Resource resource = resSet.createResource(URI.createFileURI(outputPath + model.getResourceTypesPath()));
			resource.getContents().add(restypes);
			resource.save(Collections.EMPTY_MAP);
			
			resource = resSet.createResource(URI.createFileURI(outputPath + model.getResourceLandscapePath()));
			resource.getContents().add(landscape);
			resource.save(Collections.EMPTY_MAP);
			
			resource = resSet.createResource(URI.createFileURI(outputPath + model.getRepositoryPath()));
			resource.getContents().add(repository);
			resource.save(Collections.EMPTY_MAP);
			
			resource = resSet.createResource(URI.createFileURI(outputPath + model.getSystemPath()));
			resource.getContents().add(system);
			resource.save(Collections.EMPTY_MAP);
			
			resource = resSet.createResource(URI.createFileURI(outputPath + model.getDeploymentPath()));
			resource.getContents().add(deployment);
			resource.save(Collections.EMPTY_MAP);
			
			resource = resSet.createResource(URI.createFileURI(outputPath + model.getUsageProfilePath()));
			resource.getContents().add(profile);
			resource.save(Collections.EMPTY_MAP);
		} catch(IOException e) {
			e.printStackTrace();
		}

	}
	
	private void saveLibredeConfiguration(Path outputPath, LibredeConfiguration conf) throws IOException {
		ResourceSet rset = Registry.INSTANCE.createResourceSet();
		Resource xmi = rset.createResource(URI.createFileURI(outputPath.resolve("estimation.librede").toString()));
		xmi.getContents().add(conf);
		xmi.save(Collections.EMPTY_MAP);
	}
	
	private LibredeConfiguration createLibredeConfiguration(Deployment deployment, Path dataPath) {
		LibredeConfiguration conf = ConfigurationFactory.eINSTANCE.createLibredeConfiguration();
		
		WorkloadDescriptionDerivation workload = new WorkloadDescriptionDerivation();
		workload.updateWorkloadDescription(deployment);
		conf.setWorkloadDescription(workload.getMapping().getWorkload());
		
		InputSpecificationDerivationStrategy deriveInput = new SystemInterfaceInstrumentationStrategy();
		conf.setInput(deriveInput.derive(conf.getWorkloadDescription(), workload.getMapping(), dataPath));
//		conf.setInput(ConfigurationFactory.eINSTANCE.createInputSpecification());
		
		EstimationSpecification estimation = ConfigurationFactory.eINSTANCE.createEstimationSpecification();
		estimation.setRecursive(false);
		estimation.setWindow(60);
		estimation.setStepSize(UnitsFactory.eINSTANCE.createQuantity(1.0, Time.MINUTES));
		
		
		//Tx60
		estimation.setStartTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.44316482E9, Time.SECONDS));
		estimation.setEndTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.44316842E9, Time.SECONDS));
		
//		estimation.setStartTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.438163220E9, Time.SECONDS));
//		estimation.setEndTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.438167360E9, Time.SECONDS));
		
		//Tx100
		//estimation.setStartTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.429781887E9, Time.SECONDS));
		//estimation.setEndTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.429785487E9, Time.SECONDS));
		
		EstimationApproachConfiguration approach = ConfigurationFactory.eINSTANCE.createEstimationApproachConfiguration();
		approach.setType("tools.descartes.librede.approach.ServiceDemandLawApproach");
		estimation.getApproaches().add(approach);
		
		EstimationAlgorithmConfiguration algorithm = ConfigurationFactory.eINSTANCE.createEstimationAlgorithmConfiguration();
		algorithm.setType("tools.descartes.librede.algorithm.SimpleApproximation");
		estimation.getAlgorithms().add(algorithm);
		conf.setEstimation(estimation);
		
		ValidationSpecification validation = ConfigurationFactory.eINSTANCE.createValidationSpecification();
		validation.setValidateEstimates(true);
		ValidatorConfiguration rVal = ConfigurationFactory.eINSTANCE.createValidatorConfiguration();
		rVal.setType("tools.descartes.librede.validation.ResponseTimeValidator");
		validation.getValidators().add(rVal);
		ValidatorConfiguration uVal = ConfigurationFactory.eINSTANCE.createValidatorConfiguration();
		uVal.setType("tools.descartes.librede.validation.UtilizationValidator");
		validation.getValidators().add(uVal);
		
		conf.setValidation(validation);
		
		conf.setOutput(ConfigurationFactory.eINSTANCE.createOutputSpecification());
		
		return conf;
	}

}
