package tools.descartes.prisma.parameterization;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collections;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

import edu.kit.ipd.descartes.mm.applicationlevel.functions.ExponentialDistribution;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.FunctionsFactory;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSample;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.IntSampleList;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.ProbabilityMassFunction;
import edu.kit.ipd.descartes.mm.applicationlevel.functions.RandomVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.ModelVariableCharacterizationType;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopIterationCount;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServicebehaviorFactory;
import tools.descartes.librede.Librede;
import tools.descartes.librede.LibredeResults;
import tools.descartes.librede.ResultTable;
import tools.descartes.librede.approach.IEstimationApproach;
import tools.descartes.librede.configuration.ConfigurationFactory;
import tools.descartes.librede.configuration.EstimationApproachConfiguration;
import tools.descartes.librede.configuration.EstimationSpecification;
import tools.descartes.librede.configuration.LibredeConfiguration;
import tools.descartes.librede.configuration.ResourceDemand;
import tools.descartes.librede.configuration.ValidationSpecification;
import tools.descartes.librede.configuration.ValidatorConfiguration;
import tools.descartes.librede.ipopt.java.IpoptLibrary;
import tools.descartes.librede.linalg.Vector;
import tools.descartes.librede.metrics.Aggregation;
import tools.descartes.librede.metrics.StandardMetrics;
import tools.descartes.librede.nnls.NNLSLibrary;
import tools.descartes.librede.registry.Registry;
import tools.descartes.librede.repository.MemoryObservationRepository;
import tools.descartes.librede.repository.rules.Rule;
import tools.descartes.librede.units.Quantity;
import tools.descartes.librede.units.RequestCount;
import tools.descartes.librede.units.Time;
import tools.descartes.librede.units.UnitsFactory;
import tools.descartes.prisma.core.DmlInstance;


public class Main implements IApplication {
	
	private static final int ESTIMATION_WINDOW = 15;
	
	public Object start(IApplicationContext context) throws Exception {
		Librede.initLogging();
		Librede.init();
		IpoptLibrary.init();
		NNLSLibrary.init();
		Registry.INSTANCE.registerImplementationType(IEstimationApproach.class, OptEstimationApproach.class);
		
		Path dataLocation = Paths.get("D:/Dateien/VMware/SPECjExperiments/PER-Experiments/Run_2015-10-27_16-29-30_tx60");
		Path modelLocation = Paths.get("C:/Users/Simon/Git/specjent2010/specj/specjent2010");
		DmlInstance inst = new DmlInstance(modelLocation, "specjent2010");
		
		inst.load();
		
		LibredeConfiguration conf = ConfigurationFactory.eINSTANCE.createLibredeConfiguration();
		
		WorkloadDescriptionDerivation workload = new WorkloadDescriptionDerivation();
		workload.updateWorkloadDescription(inst.getDeployment());
		conf.setWorkloadDescription(workload.getMapping().getWorkload());
		
		
		InputSpecificationDerivationStrategy deriveInput = new SystemInterfaceInstrumentationStrategy();
//		InputSpecificationDerivationStrategy deriveInput = new TierInterfaceInstrumentationStrategy();
		conf.setInput(deriveInput.derive(conf.getWorkloadDescription(), workload.getMapping(), dataLocation));
		
		EstimationSpecification estimation = ConfigurationFactory.eINSTANCE.createEstimationSpecification();
		estimation.setRecursive(false);
		estimation.setWindow(ESTIMATION_WINDOW);
		estimation.setStepSize(UnitsFactory.eINSTANCE.createQuantity(1.0, Time.MINUTES));
//		
//		
//		//Tx60
		Quantity<Time> start = UnitsFactory.eINSTANCE.createQuantity(1.445955240E9, Time.SECONDS);		
		Quantity<Time> end = UnitsFactory.eINSTANCE.createQuantity(1.445958840E9, Time.SECONDS);
		estimation.setStartTimestamp(start);
		estimation.setEndTimestamp(end);
//		
////		estimation.setStartTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.438163220E9, Time.SECONDS));
////		estimation.setEndTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.438167360E9, Time.SECONDS));
//		
//		//Tx100
//		//estimation.setStartTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.429781887E9, Time.SECONDS));
//		//estimation.setEndTimestamp(UnitsFactory.eINSTANCE.createQuantity(1.429785487E9, Time.SECONDS));
//		
//		EstimationApproachConfiguration approach = ConfigurationFactory.eINSTANCE.createEstimationApproachConfiguration();
//		approach.setType("tools.descartes.librede.approach.ServiceDemandLawApproach");
//		estimation.getApproaches().add(approach);
//		
//		EstimationAlgorithmConfiguration algorithm = ConfigurationFactory.eINSTANCE.createEstimationAlgorithmConfiguration();
//		algorithm.setType("tools.descartes.librede.algorithm.SimpleApproximation");
//		estimation.getAlgorithms().add(algorithm);
		
		EstimationApproachConfiguration approach = ConfigurationFactory.eINSTANCE.createEstimationApproachConfiguration();
		approach.setType("tools.descartes.prisma.parameterization.OptEstimationApproach");
//		approach.setType("tools.descartes.librede.approach.ServiceDemandLawApproach");
		estimation.getApproaches().add(approach);
		conf.setEstimation(estimation);
		
		ValidationSpecification validation = ConfigurationFactory.eINSTANCE.createValidationSpecification();
		validation.setValidateEstimates(true);
		ValidatorConfiguration rVal = ConfigurationFactory.eINSTANCE.createValidatorConfiguration();
		rVal.setType("tools.descartes.librede.validation.ResponseTimeValidator");
		validation.getValidators().add(rVal);
		ValidatorConfiguration uVal = ConfigurationFactory.eINSTANCE.createValidatorConfiguration();
		uVal.setType("tools.descartes.librede.validation.UtilizationValidator");
		validation.getValidators().add(uVal);
		
		conf.setValidation(validation);
		
		conf.setOutput(ConfigurationFactory.eINSTANCE.createOutputSpecification());
		
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("librede", new XMIResourceFactoryImpl());
		
		ResourceSet rset = Registry.INSTANCE.createResourceSet();
		Resource xmi = rset.createResource(URI.createFileURI(modelLocation.resolve("estimation.librede").toString()));
		xmi.getContents().add(conf);
		xmi.save(Collections.EMPTY_MAP);
		
		MemoryObservationRepository repo = new MemoryObservationRepository(conf.getWorkloadDescription());
		for (Rule<?> rule : WorkOrderServiceProxyWorkaround.getRules()) {
			repo.addRule(rule);
		}		
		repo.setCurrentTime(conf.getEstimation().getEndTimestamp());
		Librede.loadRepository(conf, repo);
		
		// Update invocation counts
		for (ExternalCall dmlCall : workload.getMapping().getDmlCalls()) {
			EObject tmp = dmlCall.eContainer().eContainer().eContainer();
			if (tmp instanceof LoopAction) {
				LoopAction loop = (LoopAction)tmp;
				tools.descartes.librede.configuration.ExternalCall libredeCall = workload.getMapping().getLibredeCall(dmlCall);
				
				double visits = repo.aggregate(StandardMetrics.VISITS, RequestCount.REQUESTS, libredeCall, Aggregation.AVERAGE, start, end);
				if (visits != visits) {
					System.err.println("No visits for " + libredeCall.getService().getName() + " -> " + libredeCall.getName());
					continue;
				}
				LoopIterationCount count = ServicebehaviorFactory.eINSTANCE.createLoopIterationCount();
				count.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
				RandomVariable var = FunctionsFactory.eINSTANCE.createRandomVariable();
				ProbabilityMassFunction func = FunctionsFactory.eINSTANCE.createProbabilityMassFunction();
				IntSampleList list = FunctionsFactory.eINSTANCE.createIntSampleList();

				// Determine invocation count: currently it does not consider the distribution
				if (Math.ceil(visits) == Math.floor(visits)) {
					IntSample samp1 = FunctionsFactory.eINSTANCE.createIntSample();
					samp1.setProbability(BigDecimal.valueOf(1.0));
					samp1.setValue(BigInteger.valueOf((long)visits));
					list.getItems().add(samp1);
				} else {									
					IntSample samp1 = FunctionsFactory.eINSTANCE.createIntSample();
					samp1.setProbability(BigDecimal.valueOf(Math.ceil(visits) - visits));
					samp1.setValue(BigInteger.valueOf((long)Math.floor(visits)));
					list.getItems().add(samp1);
					samp1 = FunctionsFactory.eINSTANCE.createIntSample();
					samp1.setProbability(BigDecimal.valueOf(1 - (Math.ceil(visits) - visits)));
					samp1.setValue(BigInteger.valueOf((long)Math.ceil(visits)));
					list.getItems().add(samp1);
				}
				func.setSamples(list);
				var.setProbFunction(func);
				count.setExplicitDescription(var);				
				loop.setLoopIterationCount(count);
				
			} else {
				System.err.println("Expected loop action");
			}
		}
		
		

		try {
			LibredeResults results = Librede.runEstimation(conf, repo);
			Object o = results.getApproaches().toArray()[0];
			ResultTable table = results.getEstimates((Class<? extends IEstimationApproach>) o, 0);
			
			Vector state = table.getLastEstimates();
			ResourceDemand[] demands = table.getStateVariables();
			for (int i = 0; i < demands.length; i++) {
				System.out.println(demands[i].getResource().getName() + "/" + demands[i].getService().getName() + ": "+ state.get(i));
			}			
			
			for (int i = 0; i < state.rows(); i++) {
				double estimate = state.get(i);
				ResourceDemand curDemand = demands[i];
				
				edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ResourceDemand dmlDemand = workload.getMapping().getDmlDemand(curDemand);
				if (dmlDemand != null) {
					dmlDemand.setCharacterization(ModelVariableCharacterizationType.EXPLICIT);
					RandomVariable var = FunctionsFactory.eINSTANCE.createRandomVariable();
					dmlDemand.setExplicitDescription(var);
					ExponentialDistribution expDist = FunctionsFactory.eINSTANCE.createExponentialDistribution();
					if (estimate != 0) {
						expDist.setRate(BigDecimal.valueOf(1 / estimate));
					} else {
						expDist.setRate(BigDecimal.valueOf(1e30));
					}
					var.setProbFunction(expDist);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		inst.storeTo(modelLocation.resolve("exp1/opt2"));
//		
//		System.out.println("T");
		return null;
	}

	@Override
	public void stop() {
		// TODO Auto-generated method stub
		
	}	
	
}