package tools.descartes.prisma.parameterization;

import java.nio.file.Path;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.repository.AssemblyContext;
import edu.kit.ipd.descartes.mm.applicationlevel.repository.Signature;
import edu.kit.ipd.descartes.mm.resourcelandscape.Container;
import tools.descartes.librede.configuration.ConfigurationFactory;
import tools.descartes.librede.configuration.DataSourceConfiguration;
import tools.descartes.librede.configuration.ExternalCall;
import tools.descartes.librede.configuration.InputSpecification;
import tools.descartes.librede.configuration.Resource;
import tools.descartes.librede.configuration.SchedulingStrategy;
import tools.descartes.librede.configuration.Service;
import tools.descartes.librede.configuration.WorkloadDescription;

public class SystemInterfaceInstrumentationStrategy extends InputSpecificationDerivationStrategy {
	
	public InputSpecification derive(WorkloadDescription workload, MappingModel mapping, Path dataLocation) {
		InputSpecification input = ConfigurationFactory.eINSTANCE.createInputSpecification();
		DataSourceConfiguration wildflyDs = createWildflyDataSource();
		input.getDataSources().add(wildflyDs);
		DataSourceConfiguration hypericDs = createHypericDataSource();
		input.getDataSources().add(hypericDs);
		
		for (Resource curResource : workload.getResources()) {
			if (curResource.getSchedulingStrategy() != SchedulingStrategy.IS) {
				Container curContainer = mapping.getContainer(curResource);
				Path filePath = dataLocation.resolve(curContainer.getName()).resolve("hyperic.csv");
				addUtilizationTrace(input, hypericDs, curResource, filePath);
			}
		}
		
		for (Service curService : workload.getServices()) {
			Container curContainer = mapping.getContainer(curService);
			AssemblyContext component= mapping.getAssemblyContext(curService);
			Signature signature = mapping.getSignature(curService);
			Path filePath = dataLocation.resolve(curContainer.getName()).resolve("operations.csv");
			addThroughputTrace(input, wildflyDs, curService, createCallFilter(component, signature), filePath);
			
			for (ExternalCall curCall : curService.getOutgoingCalls()) {
				Service calledService = curCall.getCalledService();
				Signature calledSignature = mapping.getSignature(calledService);
				AssemblyContext calledContext = mapping.getAssemblyContext(calledService);
				addThroughputTrace(input, wildflyDs, curCall,
						createCallFilter(component, signature, calledContext, calledSignature), filePath);
			}
		}
		
		List<Service> systemServices = determineSystemServices(workload);
		for (Service curService : systemServices) {
			Container curContainer = mapping.getContainer(curService);
			AssemblyContext component = mapping.getAssemblyContext(curService);
			Signature signature = mapping.getSignature(curService);
			Path filePath = dataLocation.resolve(curContainer.getName()).resolve("operations.csv");
			addResponseTimeTrace(input, wildflyDs, curService, createCallFilter(component, signature), filePath);
		}		
		return input;
	}

}
